;; GroupChat.scm
;;
;; (define p2p (make-p2p-chat MYNAME COLOR))
;;
;; (p2p 'get 'panel)                 returns the panel containing the chatting area 

;; (p2p 'get 'group-client))         returns/sets the group client object
;; (p2p 'set 'group-client GC))

;; (p2p 'get 'name))                 returns/sets the current client's name
;; (p2p 'set 'name Name))            default is set by call to make-p2p-group-chat

;; (p2p 'get 'channel))              returns/sets the current client's name
;; (p2p 'set 'channel Channel))      default is 'groupchat
;;
;; Refactor this to use an external highlight manager
;; the manager should keep track of user colors based on user id
;; Just as with the GTAC. Ideally, highlight color management should
;; take place in the highlight module and be shared by the GTAC and GroupChat
;;

(environment-import "jlib/Swing.scm")

   (define printdebug (lambda (type . R)
      (if (equal? type 'error)
          (display {PRINTDEBUG::[type]:[(.toString R)]\n}))))

   (define make-p2p-group-chat (lambda R
      ;; state variables for the chat panel
    (let (
      (Name "anonymous")
      (chat-buf (java.lang.StringBuffer.))
      (Color (java.awt.Color. 255 0 0))
      (inchannel 'groupchatchannel)
      (rgbcolorlist (list 255 0 0))
      (GC (lambda R #t))             ;; GroupClient
      (tt (maketagger))
         )


      (define the-panel
        (border
          (center
            (scrollpane
              (tt "chatarea" 
                (textarea 3 40 "\n" (color 255 235 215) 
                  (lambda(this) 
;                    (.addKeyListener this silencer)
                    (.setEditable this #f)
                  )
                ) 
              )
            )
          )
          (south
           (border 
             (west (tt "uchoice" (apply choice 
                     (cons "ALL" (GC 'get 'users-in-my-group)))));;(label ">>"))
             (center
              (tt "chatline" (textfield "" 1)))))))


      (define chat-action
         (action (lambda(e)
                (GC 'send inchannel Name (readstring (tt "chatline"))
		    rgbcolorlist (readstring (tt "uchoice")))
                (writestring (tt "chatline") ""))))


      (define (update-users)
	(.removeAllItems (tt "uchoice"))
	(map (lambda (x) (.addItem (tt "uchoice") x))
	     (cons "ALL" (GC 'get 'users-in-my-group))))

      (define groupserver-listeners
	(lambda (cmd . R)
	  (case (first R)
	    ;;R= ('userleft threadcount username #peers-left-in-group)
	    ((newuser userleft) (update-users)))))

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ;; This listense to messages sent as follows:
       ;; (GC 'send inchannel Name (readstring (tt "chatline")) rgbcolorlist)               
      (define chat-listener 
       (let* 
         ((endPos (.getEndPosition (.getDocument (tt "chatarea"))))
          (ta (tt "chatarea"))
          (highlighter (.getHighlighter ta)))
        (lambda R   
          (define username (second R))
          (define raw-text (third R))
	  (define recipient (fifth R))
          (define text (string-append username
				      (if (equal? recipient "ALL")
					  ""
					  {->[recipient]})
				      ":\t   " raw-text "    \n"))
          (define currentline (.getLineOfOffset ta (+ -1 (.getOffset endPos))))
	  (if (and
	       (not (equal? "" (.trim raw-text)))
	       (or (equal? recipient Name) (equal? username Name)
		   (equal? recipient "ALL")))
	      (begin
		(if (equal? recipient "ALL")
		    (.append chat-buf text))
		(.append ta text) 
		(.addHighlight highlighter 
			       (.getLineStartOffset ta currentline) 
			       (+ -2 (.getLineEndOffset ta currentline)) 
			       (getHighlight (list-ref R 3)))
		(.setCaretPosition ta (+ -1 (.getOffset endPos)))))
     )))

      ;; this looks up a highlight painter for a color list '(R G B)
      ;; and creates one if it hasn't already cached it.
      (define getHighlight 
       (let ((highlights (java.util.Hashtable.)))
        (lambda (x)
         (let ((h (.get highlights x)))
             (if (not (equal? #null h)) h
                (begin
                   (.put highlights x
                       (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter. (apply color x)))
                   (.get highlights x)))))))

      (define msghandler
        (lambda (cmd . args)
           (case cmd
             ((get)
              (case (first args)
                 ((panel) the-panel)
                 ((chatarea) (tt "chatarea"))
		 ((chat-text) (.toString chat-buf))
                 ((name) Name)
                 ((inchannel) inchannel)
                 ((group-client) GC)))

	     ((append-chat) (.append chat-buf (first args)))
	     ((set-chat) (.setLength chat-buf 0)
	                 (.append chat-buf (first args)))

	     ((setall)     ;; this allows multiple sets: (P 'setall 'name "Tim" 'color blue)
	      (let loop ((L args)) 
		(if (null? L) #t
                    (begin 
		      (msghandler 'set (first L) (second L)) 
		      (loop (rest (rest L)))))))

             ((setfont)
                  (.setFont (tt "chatarea") (first args))
                  (.setFont (tt "chatline") (first args)))

             ((set)
   	           (case (first args)
                  ((name)
                     (begin
                     (set! Name (second args))))
                  ((color)
                     (set! Color (second args))
                     (set! rgbcolorlist (list (.getRed Color) (.getGreen Color) (.getBlue Color))))
                  ((inchannel) (set! inchannel (second args)))

                  ((group-client)
                   (begin
                     (GC 'logout)
               	     (set! GC (second args))
                     (GC 'add-listener inchannel chat-listener)
		     (GC 'add-listener 'GROUPSERVER groupserver-listeners)
                     (update-users)
                 ))))



             (else (printdebug 'error (list "ERROR" cmd args)))
	   )))

     (apply msghandler (cons 'setall R)) ;; interpret parameters as setting specifiers
;     (.addKeyListener (tt "chatarea") (jsint.Listener. (lambda(e) (.consume e)))) ;; don't allow typing in the chat area
     (.setBackground (tt "chatarea") white)
     (.addActionListener (tt "chatline") chat-action)

;      (GC 'add-listener 'GROUPSERVER (lambda (cmd . R)
;        (case (first R)
; 	 ((userleft) ;;R= ('userleft threadcount username #peers-left-in-group)
; 	  (GC 'send inchannel Name {Left Grewp ([(Date.)])} rgbcolorlist))
; 	 ((newuser) ;;R= ('newuser threadcount username)
; 	  (GC 'send inchannel Name {Joined Grewp ([(Date.)])} rgbcolorlist))
;        (else #t))))

;;;;;;;;;;(GC 'send inchannel Name (readstring (tt "chatline")) rgbcolorlist)


     msghandler)
))










