;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; HighlightManager.scm
;;  This is a simple module for maintaining a set of highlights
;;
;; API
;;  (define H (make-Highlightmanager ta))
;;
;;  (H 'highlight insoffset the-id len)   --- highlight around the insoffset for the specified id
;;  (H 'set-color id (r g b a))
;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define (make-HighlightManager the-textarea)

  (define msg-handler (lambda (cmd . R)
;;;    (display (list '******hm cmd R))(newline)
    (case cmd
      ((highlight) (apply do-highlight R))
      ((set-color) (set-color (first R) (second R) (third R)))
      (else (display (list "unknown command to HighlightManager" cmd R))(newline) #f))))

  (define highlighter (.getHighlighter the-textarea))
  (define the-doc (.getDocument the-textarea))
  (define highlight-table (Hashtable.))

  (define (set-color id the-color offset)
    ;;this hopefully solves the highlight bug
    (let ((x (.get highlight-table id)))
      (if (not (equal? x #null))
	  (.removeHighlight highlighter x))
      (.put highlight-table id  
          (create-new-highlight the-color (max 0 (- offset 10)) offset))))

  (define (move-highlight id start end)
    (let ((x (.get highlight-table id)))
      (if (not (equal? x #null))
	  (tryCatch
	    (.changeHighlight highlighter x start end)
	    (lambda(e) (display (list "ERROR in highlight" id start end)) (newline))
	  ))))


  (define (do-highlight pos id)
    (move-highlight id (max 0 (- pos 10)) pos)
   )


  (define (create-new-highlight the-color start end)
    (tryCatch
      (let* ((hlpainter (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter. 
                             (apply java.awt.Color. the-color)))
	     (h (.addHighlight highlighter start end hlpainter)))
         h
       )
      (lambda(e) (display (list {ERROR in create-new-highlight} the-color start end e)(newline)))
    ))

  msg-handler
      
)
