;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Whiteboard.scm
;;
;; This file contains a whiteboard groupware component
;;
;; Use:
;;  (define W (make-whiteboard GROUP_CLIENT COLOR))
;;
;; Methods:
;;  (W 'get 'gui)
;;      -- this is the panel containing the swing canvas and the controls (line, free, text, rect)
;;  (W 'get 'group-client) (W 'set 'group-client)
;;       -- this is the group client used for all network communication
;;  (W 'get 'color)  (W 'set 'color)
;;       -- this is the color used for self-drawn lines, etc.
;;  (W 'get 'channel)  (W 'set 'channel)
;;       -- this is the key on which the whiteboard listens
;;
;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(jlib.Swing.load)

(define (make-whiteboard N COLOR)
  (make-logging-whiteboard N COLOR 
     (lambda R #t)))

(define (make-logging-whiteboard Group-client COLOR logger)

 (define channel 'wb)
 (define colorRGBlist (list (.getRed COLOR) (.getGreen COLOR) (.getBlue COLOR)))
 (define (list->color L) (apply color L))
 (define (string->color S)  (eval (string->expr S)))

 (define t (maketagger))

 (define (add-logger F)
  (lambda R
    (apply logger (cons 'wb R))
    (apply F R)))

 (define N (add-logger Group-client))

 (define hiddenpanel
    (grid 6 1   
	  (row 'none 'center (t "type" (textfield "Free" 15)))
	  (row 'none 'center (label "Enter text here") )

          (row (t "lastx" (textfield "" 10))  (t "lasty" (textfield "" 10)))))


 (define the-panel
  (border
   (north
     (row (label (string-append "color:" colorRGBlist) COLOR)))
   (west
    (grid 1 1

	 (col 'none 'north
	(grid 4 1
	      (button "Free" (action (lambda(e) 
		(writeexpr (t "type") "Free"))))
	      (button "Text" (action (lambda(e) 
                (.setEnabled (t "text") #t)
		(writeexpr (t "type") "Text"))))
	      (button "Line" (action (lambda(e)
		 (writeexpr (t "type") "Line"))))
	      (button "Rect" (action (lambda(e)
		 (writeexpr (t "type") "Rect")))))))) 
  (south 
    (col
	      (button "Clear" (action (lambda(e) 
		(N 'send channel 'erase))))

      (t "text" (textfield "" 15 (action (lambda(e) 
             (drawToGraphics (.bufferg$ (t "board")) laststring (readstring (t "text")) COLOR lastx lasty)
             (.repaint (t "board"))

             (set! laststring "")
             (writestring (t "text") "")
             (.setEnabled (t "text") #f)
             (writestring (t "type") "Free")))))))
  (center
   (t "board" (groupscheme.platform.wrapper.JSchemePanel. 150 150)))
   ;(t "board" (javax.swing.JPanel.)))
    ))


(define lastx 0)
(define lasty 0)
(define clastx 0)
(define clasty 0)
(define laststring "")



(define (mouseaction e)
  (if not-initialized (begin (init-board) (set! not-initialized #f)))


 (case (readexpr (t "type"))
 ((Text)
      (if (equal? (.getID e) java.awt.event.MouseEvent.MOUSE_PRESSED$)
        (begin
          (writestring (t "text") "")
          (set! laststring "")
          (set! lastx (.getX e))
          (set! lasty (.getY e))
          (writeexpr (t "lastx") lastx) 
          (writeexpr (t "lasty") lasty) 
          (.requestFocus (t "text")))))
 ((Free)
       (begin
         (if (equal? (.getID e) java.awt.event.MouseEvent.MOUSE_DRAGGED$)
            (begin      
                (N 'send channel 'free 'nocursor lastx lasty (.getX e) (.getY e) colorRGBlist)

                ;; these two are only needed if the machine is really slow as the "repaint"\
                ;; command at the bottom will immediately redraw the lines on the window anyway

                ;; note we first draw in black and send the event to the group server
                ;; when the even comes back the listener will redraw the line in the current color!
                ;; this allows you to see lag but not be annoyed by it!!!

		(.setColor (.bufferg$ (t "board")) gray)
		(.drawLine (.bufferg$ (t "board")) lastx lasty (.getX e) (.getY e))
                (.repaint (t "board"))
          ))

         (set! lastx (.getX e))
         (set! lasty (.getY e))
        ))
 ((Line)
       (begin       
        (if (equal? (.getID e) java.awt.event.MouseEvent.MOUSE_PRESSED$)
 	  (begin
 	    (set! lastx (.getX e))
 	    (set! lasty (.getY e))))

        (if (equal? (.getID e) java.awt.event.MouseEvent.MOUSE_RELEASED$)
 	  (begin
               (N 'send channel 'line 'nocursor lastx lasty (.getX e) (.getY e) colorRGBlist)
	       (.setColor (.bufferg$ (t "board")) COLOR)
	       (.drawLine (.bufferg$ (t "board")) lastx lasty (.getX e) (.getY e))
	       (.repaint (t "board"))))
	      
        ))
  ((Rect)
        (if (equal? (.getID e) java.awt.event.MouseEvent.MOUSE_PRESSED$)
  	  (begin
  	    (set! lastx (.getX e))
  	    (set! lasty (.getY e))))

        (if (equal? (.getID e) java.awt.event.MouseEvent.MOUSE_RELEASED$)
  	  (begin
	    (let ((width (- (.getX e) lastx))
		  (height (- (.getY e) lasty)))
	      (N 'send channel 'rect 'nocursor lastx lasty width height colorRGBlist)
	      (.setColor (.bufferg$ (t "board")) COLOR)
	      (.drawRect (.bufferg$ (t "board")) lastx lasty width height)
	      (.repaint (t "board"))
            )
        )))
))

(define (drawToGraphics graphics laststring currentstring color lastx lasty)
    (.setColor graphics white)
    (.drawString graphics laststring lastx lasty)
    (.setColor graphics color)
    (.drawString graphics  currentstring lastx lasty)
)

(define (draw-string laststring currentstring color lastx lasty)
    (drawToGraphics (.getGraphics (t "board")) laststring currentstring color lastx lasty)
    (drawToGraphics (.bufferg$ (t "board")) laststring currentstring color lastx lasty)
    (.repaint (t "board"))
)

 (define jnk3 (begin (display "wb3")(newline)))

  (define not-initialized #t)

  (define (init-board)
    (.setColor (.bufferg$ (t "board")) white)
    (.fillRect (.bufferg$ (t "board")) 0 0 (.getWidth (t "board")) (.getHeight (t "board")))
    (.repaint (t "board")))


;(.setBackground (t "board") white)

(.addMouseMotionListener (t "board")  (jsint.Listener. mouseaction))
(.addMouseListener       (t "board")  (jsint.Listener. mouseaction))
(.addKeyListener (t "text") (jsint.Listener. (lambda(e)
  (let (
	;(g (.getGraphics (t "board")))
        (s (readstring (t "text")))
       )
    (drawToGraphics (.getGraphics (t "board")) laststring s COLOR lastx lasty)
    (writeexpr (t "lastx") lastx)
    (writeexpr (t "lasty") lasty)
    (set! laststring s)
    (N 'send channel 'text 'nocursor lastx lasty laststring s colorRGBlist)
))))






   ;;; R has form (abc lastx lasty x y)
(N 'add-listener channel 
(add-logger 
 (lambda R
  (if not-initialized (begin (init-board) (set! not-initialized #f)))
  (case (second R)

   ((erase)
      (.setColor (.bufferg$ (t "board")) white)
      (.fillRect (.bufferg$ (t "board")) 0 0 (.getWidth (t "board")) (.getHeight (t "board")))
      (.repaint (t "board")))


   ((text) 
    (let (
	  ;(g (.getGraphics (t "board")))
          (lastx (list-ref R 3))
          (lasty (list-ref R 4))
          (laststring (list-ref R 5))
          (current (list-ref R 6))
          (the-color (list->color (list-ref R 7)))
         )
     (drawToGraphics (.bufferg$ (t "board")) laststring current the-color lastx lasty) 
     (.repaint (t "board"))
            
   ))

  ((free)
    (let (
	  ;(g (.getGraphics (t "board")))
	  )
      (.setColor (.bufferg$ (t "board")) (list->color (list-ref R 7)))
      (.drawLine (.bufferg$ (t "board"))  (list-ref R 3) (list-ref R 4) (list-ref R 5) (list-ref R 6))
      (.repaint (t "board"))
   ))

  ((line)
   (let (
	 ;(g (.getGraphics (t "board")))
	 )
     (.setColor (.bufferg$ (t "board")) (list->color (list-ref R 7)))
     (.drawLine (.bufferg$ (t "board")) (list-ref R 3) (list-ref R 4) (list-ref R 5) (list-ref R 6))
     (.repaint (t "board"))))

   ((rect)
   (let (
	 ;(g (.getGraphics (t "board")))
	 )
     (.setColor (.bufferg$ (t "board")) (list->color (list-ref R 7)))
     (.drawRect (.bufferg$ (t "board")) (list-ref R 3) (list-ref R 4) (list-ref R 5) (list-ref R 6))
     (.repaint (t "board"))))

   ((cursor)
    (let (
	  (g (.getGraphics (t "board")))
	  )
         (.setXORMode g white)
         (.setColor g(.darker (list->color (list-ref R 7))))
         (.drawOval g (+ -5 (list-ref R 3)) (+ -5 (list-ref R 4)) 10 10)
         (.drawOval g (+ -5 (list-ref R 5)) (+ -5 (list-ref R 6)) 10 10)
	 (.setPaintMode g)
        ))
         

   ((startend-cursor)
    (let (
	  (g (.getGraphics (t "board")))
	  )
         (.setXORMode g white)
         (.setColor g (.darker (list->color (list-ref R 5))))
         (.drawOval g (+ -5 (list-ref R 3)) (+ -5 (list-ref R 4)) 10 10)
         (.setPaintMode g)
       ))

  (else (display (list "we need some integration!!!" R))(newline))
 ) 
)))


    (lambda (cmd . args)
       (case cmd
         ((get) 
           (case (first args)
             ((gui) the-panel)
             ((group-client) N)
             ((color) COLOR)
             ((channel) channel)
             (else (list 'error "unknown args for get: " args))))
         ((set) 
           (case (first args)
             ((group-client) (set! N (second args)))
             ((color) (set! COLOR (second args)))
             ((channel) (set! channel (second args)))
             (else (list 'error "unknown args for get: " args))))
         (else
           (list 'error "unknown command:" cmd args))))
)






























