;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
{
 GrewpPanel.scm

 This module defines the procedure
   (make-grewp-panel host port user pw mainwin JT base)
 where host/port/user/pw are the data needed to connect to 
 a groupserver selected earlier by the user, and mainwin/JT 
 are the window/tabbed pane that will contain this widget.

 A GrewpPanel consists of four main components and three main
 actions. The four main components are:
 *userlist*
 *placelist*
 *grewpletlist*
 *GChat*

 The three main actions are:
 *join*
 *new*
 *Change Color*

 *base* is the interface used to manage the grewplets this 
 GrewpPanel supports

 This procedure creates a GREWP panel which keeps track of the users
 and documents being shared and allows users to create and/or
 join shared edit sesions, as well as to change their color.
 User's who don't like a color can close the Doc window,
 change the GREWP color and open another Doc window...
}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(environment-import "groupscheme/platform/util/Util.scm" "U:")
(environment-import "groupscheme/platform/util/FileUtil.scm" "FileU:")
(environment-import "groupscheme/platform/GrewpHelp.scm" "GH:")
(environment-import "groupscheme/platform/GS/GroupChat.scm" "GChat:")
(environment-import "jlib/Swing.scm")

(define (make-grewp-panel host port user pw RGC GS mainwin JT base)

  ;;locked-list? is for the critical moment of selection response.
  ;;A placelist selection triggers a userlist selection and vice
  ;;versa so not locking can end up in a selection loop
  (define locked-list? #f)
  (define (lock-select jlist values)
    (set! locked-list? #t)
    (U:select-values jlist values)
    (set! locked-list? #f))

  (define userlist (jlist (lambda(x)
    (if (not locked-list?)
	(lock-select placelist 
	   (map (lambda(place)
		  (if (member x (RGC 'get 'users-in-group place)) place))
		(RGC 'get 'groups)))))))

  (define placelist (jlist (lambda(x)
    (if (not locked-list?)
	(lock-select userlist (RGC 'get 'users-in-group x)))
    (if (equal? #null x)
	(.setEnabled join-button #f)
	(.setEnabled join-button #t)))))

  (define grewpletlist (jlist (lambda(x)
    (if (member x (base 'names))
	(.setEnabled new-button #t)
	(.setEnabled new-button #f)))))


  (define ucolor (color (U:rand 190 240) (U:rand 190 240) (U:rand 190 240)))

  (define (unique name type)
    (set! name (if (string? name) (.trim name) #null))
    (cond ((equal? name #null) #f)
	  ((or (equal? name "") (member {[type]:[name]} (RGC 'get 'groups)))
	     (unique (javax.swing.JOptionPane.showInputDialog
		      {[{<html>The name "[name]" is either in use or illegal!}
			{<br>Please try again.</html>}]})
		     type))
	  (else name)))

  (define (add-grewplet-gui type name)
    (let ((pane (base 'call type 'gui RGC ucolor name pw mainwin JT)))
      (.add JT name pane)
      (.setSelectedComponent JT pane)
      (RGC 'send 'rollcall)))

  (define (join-grewplet name)
    (let ((index (U:getTabIndex name JT))
	  (strpos (.indexOf name ":")))
      (cond ((equal? name #null)  (.setEnabled join-button #f))
	    ((> index -1)         (.setSelectedIndex JT index))
	    (else                 
	     (let ((type (if (!= -1 strpos)
			     (.substring name 0 strpos)
			     #null)))
	       (if (member type (base 'names))
		   (add-grewplet-gui type name)
		   (U:error {No support for Grewp Type [type]} the-panel)))))))

  (define (make-grewplet type name)
    (if (member type (base 'names))
	(if (base 'call type 'remote-server?)
	    (RGC 'send 'create-grewplet-server
		 type (list host port user) name)
	    (add-grewplet-gui type name))
	(U:error {No support for Grewp Type [type]} the-panel)))

  (define (make-menu)
    (let ((fmenu (menu "File"))
	  (gmenu (menu "Grewp"))
	  (hmenu (menu "Help"))
	  (fc (javax.swing.JFileChooser. (base 'dir))))
      ;;JFileChooser.DIRECTORIES_ONLY || FILES_ONLY
      (.setFileSelectionMode fc 
	  javax.swing.JFileChooser.FILES_AND_DIRECTORIES$)
      (.add fmenu (menuitem "Save Chat"
         (action (lambda(e) (FileU:static-save (GChat 'get 'chatarea))))))
      (.add fmenu (menuitem "Refresh"
         (action (lambda(e) (RGC 'send 'rollcall)))))
      (.add fmenu (javax.swing.JSeparator.))
      (.add fmenu (menuitem "Exit" 
         (action (lambda(e)
	    (.hide mainwin)
	    (RGC 'send 'rollcall 'delay)
	    (RGC 'logout)
	    (tryCatch (if GS (GS 'quit))
		      (lambda(e) (display {\nJust quit [GS]\n\n})))))))
      (.add gmenu (menuitem "Add Type(s)"
	 (action (lambda(e)
	   (if (= (.showOpenDialog fc mainwin)
		  javax.swing.JFileChooser.APPROVE_OPTION$)
	       (base 'add (java.io.File. (.getCurrentDirectory fc)
					 (.getName (.getSelectedFile fc)))))
	   (jlistwrite grewpletlist (base 'names))))))
      (.add gmenu (menuitem "Import Type"
	 (action(lambda(e)
           (base 'add (javax.swing.JOptionPane.showInputDialog
		       {Please enter the URL of the plugin:}))
	   (jlistwrite grewpletlist (base 'names))))))
      (.add hmenu (menuitem "About" (action (lambda(e) (GH:show-about)))))
      (menubar fmenu gmenu hmenu)))

  (define colorbutton
    (U:colorbutton "Change Color" ucolor
      (lambda (e)
	(set! ucolor (javax.swing.JColorChooser.showDialog 
		      the-panel "Pick your user color" ucolor))
	(if (not (.isInstance java.awt.Color.class ucolor))
	    (set! ucolor (.getBackground (.getSource e)))
	    (begin (GChat 'set 'color ucolor)
		   (.setBackground (.getSource e) ucolor)
		   (.setBackground (GChat 'get 'chatarea) ucolor))))))

  (define join-button (button "Join" (action (lambda (e)
    (for-each join-grewplet
	      (array->list (.getSelectedValues placelist)))))
     (lambda (this) (.setEnabled this #f))))

  (define new-button (button "New" 
      (action (lambda (e)
        (let*
	    ((type (.getSelectedValue grewpletlist))
	     (name (if (not (equal? type "**Grewplet Type(s)**"))
		       (unique
			(javax.swing.JOptionPane.showInputDialog
			 "What do you want to call your Grewp Place?")
			type))))
	  (if name (make-grewplet type {[type]:[name]})))))
      (lambda (this) (.setEnabled this #f))))

  (define GChat 
    (GChat:make-p2p-group-chat 'name user 'color ucolor 'mode 'gui))
 
  (define core-panel
    (vsplit 0.5
       (grid 1 3
	  (splitcol 1 40
	      (row 'none 'west (label "Connected Users" (HelveticaBold 20)))
	      (scrollpane userlist))
	  (splitcol 1 40
	      (row 'none 'center (label "Grewp Places" (HelveticaBold 20)))
	      (scrollpane placelist))
	  (splitcol 1 40
	      (row 'none 'east (label "Grewp Types" (HelveticaBold 20)))
	      (scrollpane grewpletlist)))
       (GChat 'get 'panel)))


  ;; here we define the panel that will be returned by this procedure ..
  (define the-panel
   (border
    (north (make-menu))
    (center core-panel)
    (south 
     (col
      (row (row 'none 'west colorbutton)
	   (row 'none 'east (grid 1 2 join-button new-button)))
      (row 'none 'center
	   (label {[{Connected to '[(RGC 'get 'group)]' }
		    {as '[user]'   ---Location [host]:[port]:[pw]}]}
		  (HelveticaBold 16)))
   )))) ;;close col/south/border/define


  ;; Setup Preferences
  (U:add-multi-click placelist (lambda (e) (.doClick join-button)) 2)
  (U:add-multi-click grewpletlist (lambda (e) (.doClick new-button)) 2)
  (jlistwrite grewpletlist (base 'names))
  (U:grewp-chat-setup RGC GChat ucolor
     {*** Welcome to the Global chat space for the grewp\n})
  (.setDividerLocation core-panel 0.5)

  ;; Add Listeners
  (RGC 'add-listener 'place-update 
    (lambda (update msg) 
      (jlistwrite placelist msg)))

  (RGC 'add-listener 'user-update
    (lambda (update msg)
      (jlistwrite userlist msg)))
  
  ;; if a grewplet-server has been created for this user, then create
  ;; a grewplet panel of that type and add it to the JTabbedPane
  (RGC 'add-listener 'grewplet-server-created
    (lambda (cmd type id filename)
      (if (equal? id (list host port user))
	  (add-grewplet-gui type filename))))


  (RGC 'send 'rollcall)
  the-panel
)