;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
{
 NetworkPanel.scm

 This module defines the procedure 
   (make-network-panel JT mainwin)
 where *JT* and *mainwin* are passed along to the GrewpPanel
 that is created when a user either:
 *Connect to our GREWP*
    This option connects the user to a GREWP we (should) have 
    up and running all the time so that users can have a global
    community and use the software without having to start their
    own GREWP or knowing of one to join.
 *Connect to another GREWP*
    This option is for users that know the URL, port, and session
    name to a given existing GREWP that they would like to join.
 *Start a local GREWP*
    This option is for users that would like to host a GREWP on
    their computer thereby becoming an existing GREWP that users
    can join providing they know the URL/port/SessionName that 
    was chosen to identify the GREWP.

 Note: The error of connecting to a non-existing GREWP is handled
       but takes too long to unravel and gives the impression the
       software has frozen.
}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(environment-import "groupscheme/platform/GS/GroupServer.scm" "GS:")
(environment-import "groupscheme/platform/GS/GroupClient.scm" "GC:")
(environment-import "groupscheme/platform/GS/SendLine.scm" "SL:")
(environment-import "groupscheme/platform/util/Util.scm" "U:")
(environment-import "groupscheme/platform/util/PluginBase.scm" "PB:")
(environment-import "groupscheme/platform/RendezvousServer.scm" "RS:")
(environment-import "groupscheme/platform/GrewpHelp.scm" "GH:")
(environment-import "groupscheme/platform/GrewpPanel.scm" "GP:")
(environment-import "jlib/Swing.scm")

;;NOTE:
;;SGC seems to die every so often and i don't know why...
;;When it does die it can't receive messages from the rendezvous server
;;so no updates are displayed

(define (make-rendezvous-panel JT network-win)
  (define tag (maketagger))
  (define (show-alert msg)
    (let ((w (window "GrewpEdit"
	       (label (HelveticaBold 28)
		      {<html><body color="blue">[msg]</body></html>}))))
      (.pack w) (.show w)
      w))

  (define local-ip 
    (tryCatch
     (.getHostAddress (java.net.InetAddress.getLocalHost))
     (lambda(e) "127.0.0.1")))

  (define (local-check) #t)
;     (if (not (.isAnyLocalAddress (java.net.InetAddress.getByName local-ip)))
; 	(U:error {Your IP address '[local-ip]' is some form of local address!
; This will block communication with our rendezvous server.
; You can (if you know it) set your global IP, menu click
; 'Options' -> 'Set Local Host' or try starting your own
; rendezvous server 'Options' -> 'Rendezvous Server'. }
; 		 network-win)))

  (define PW (if U:web-vars-loaded? U:WV:rendezvousPW U:SV:rendezvousPW))
  (define sHost (if U:web-vars-loaded?
		    U:WV:rendezvousHost U:SV:rendezvousHost))
  (define sPort (if U:web-vars-loaded? 
		    U:WV:rendezvousPort U:SV:rendezvousPort))
  (define PLUGINS (if U:web-vars-loaded? 
		      (append U:WV:plugins U:SV:localPlugins)
		      U:SV:localPlugins))

  (define get-port
    (let* ((minPort (if U:web-vars-loaded? U:WV:minPort U:SV:minPort))
	   (maxPort (if U:web-vars-loaded? U:WV:maxPort U:SV:maxPort))
	   (port minPort))
      (lambda()
	(set! port (+ port 10))
	(if (> port maxPort) (set! port minPort))
	port)))

  (define SGC ())

  (define connected-to-rendezvous-server? #t)

  (define (rendezvous-server? host port)
    (tryCatch 
     (let* ((server? (SL:send-line host port "rendezvous-server?\n")))
       (set! connected-to-rendezvous-server? (and (boolean? server?) server?)))
     (lambda(e) (set! connected-to-rendezvous-server? #f))))

  (define (valid-locale? name)
    (if connected-to-rendezvous-server?
	(let ((locale (SL:send-line sHost sPort {locale\n[name]\n})))
	  (not (equal? #null locale)))))

  (define (rendezvous-server-connect host port)
    (define win (show-alert {Trying to connect Rendezvous Server:<br>
			     @ [host]:[port]}))
    (set! sHost host)
    (set! sPort port)
    (tryCatch
     (if (rendezvous-server? host port)
	 (begin
	   (tryCatch
	    (if (not (null? SGC))
		(SGC 'logout))
	    (lambda(e) (display {Could not logout of [SGC]\n})))
	   (set! SGC (GC:make-group-client "RendezvousClient" "*GREWPSERVER*"
				 sHost sPort "GREWPscheme"))
	   (writestring (tag "status")
	       {Connected to rendezvous server @ [sHost]:[sPort]})
	   (add-rendezvous-listener SGC)
	   (jlistwrite (tag "groups")
		       (SL:send-line sHost sPort "locale-names\n"))
	   (.hide win))
	 (throw {[sHost]:[sPort] is not a rendezvous server}))
     (lambda(e)
       (.hide win)
       (writestring (tag "status")
	   {Not connected to a rendezvous server!})
       (U:error {Could not connect to rendezvous server @ [sHost]:[sPort]}
		network-win)
       (display {Error: [e]\n}))))

  (define (rendezvous-server-start port)
    (define win (show-alert {Trying to connect Rendezvous Server:<br>
			     @ [local-ip]:[port]}))
    (set! sHost local-ip)
    (set! sPort port)
    (tryCatch
     (begin (RS:make-rendezvous-server sPort)
	    (rendezvous-server-connect sHost sPort)
	    (.hide win))
;       (let* ((gs (RS:make-rendezvous-server sPort))
; 	     (pane (border 
; 		    (center (gs 'get 'gui))
; 		    (south (button "Shutdown Rendezvous Server"
; 		      (action (lambda (e)
; 			(.removeTabAt JT
; 			   (.indexOfTab JT "My Rendezvous Server"))
; 			(gs 'quit)			
; 			(.setSelectedIndex JT 0)
; 			(.pack network-win)))))) ) )
; 	(rendezvous-server-connect sHost sPort)
; 	(.add JT "My Rendezvous Server" pane)
; 	(.hide win))
      (lambda(e)
	(.hide win)
	(U:error {Could not start rendezvous server on: [sHost]:[sPort]}
		 network-win)
	(display {Error: [e]\n}))))

  ;; This is used to verify that *name* is not already 
  ;; connected to host/port
  (define (unique name host port type)
    (set! name (if (string? name) (.trim name) #null))
;;    (tryCatch
     (cond ((equal? name #null) #f)
	   ((or (equal? name "") (.equalsIgnoreCase "system" name)
		(member name (SL:send-line host port type)))
	    (unique (javax.swing.JOptionPane.showInputDialog
		     network-win
		     {[{<html>The name "[name]" is either in use or }
		       {illegal!<br>Please try a different name.</html>}]})
		    host port type))
	   (else name))
;;     (lamba(e) )
         )

  ;; The interface to all default grewplets loaded from directory below
  (define dir (java.io.File. "groupscheme/platform/grewplet/"))
  (define base (PB:make-plugin-base dir "grewplet"))
  (define jar-OR-webstart
    (if (not (.exists dir)) ;; then must be running as jar or web start
	(map (lambda(x) (base 'add x)) PLUGINS)))

  
  (define (password-verifier pw)
    (lambda (password user group)
      (equal? password pw)))

  (define (add-rendezvous-listener G)
    (G 'add-listener 'rendezvous (lambda (cmd . R)
       (case (first R)
	 ((rollcall)
	  (jlistwrite (tag "groups") (second R)))
	 (else (display {Can't handle [(first R)]\n}))))))

  ;; This adds a listener that simply sends out a list of connected
  ;; users on the channel 'user-update and a list of started grewp
  ;; places on the channel 'place-update
  (define (add-registrar-listener G)
    ;;(G 'add-super-listener (lambda R (display (list 'registrar R))(newline)))
    (G 'add-listener 'rollcall (lambda (cmd . R)
       (if (equal? (first R) 'delay)
	   (.start (java.lang.Thread. (lambda()
		     (java.lang.Thread.sleep 5000L)
		     (G 'send 'rollcall))))
	   (begin
	     (G 'send 'user-update (G 'get 'users))
	     (G 'send 'place-update (G 'get 'groups)))))))

  ;; This adds a listener that allow users to request that the host
  ;; of their GREWP start a specific type of grewplet server. After 
  ;; starting the server the host sends the requesting user a 
  ;; 'grewplet-server-created message as a safety ok for creating 
  ;; that type of the grewplet.
  (define (add-grewplet-server-listener host port pw RGC F)
    (RGC 'add-listener 'create-grewplet-server
	 (lambda (cmd type id grewpname)
	   (if (F id)
	       (begin
		 (display {StartServer: [type]\n})
		 (base 'call type 'start-server host port grewpname pw)
		 (RGC 'send 'grewplet-server-created type id grewpname))))))

  ;; This is the core procedure of the NetworkPanel. In any case we make
  ;; a group client, then make a grewp panel for that client. However,
  ;; in the case when the paramerter *server?* is true, we start a 
  ;; GroupServer before we do so and we also add the registrar and 
  ;; grewplet-server listeners to enable GREWP hosting capabilities.
  (define (grewp-init server? name group host port pw)
    (let* ((gs (if server?
			 (GS:make-group-server port (password-verifier pw))))
	   (user (unique name host port "users\n"))
	   (_JT (javax.swing.JTabbedPane.))
	   (_mainwin (window (.getTitle network-win) (grid 1 1 _JT))))
      (if user  ;;then not canceled
	  (let* ((RGC (GC:make-group-client user group host port pw))
		 (pane (GP:make-grewp-panel host port user pw RGC gs
					    _mainwin _JT base)))
	    (.add _JT group pane)
	    (cond (server?
		   (add-registrar-listener RGC)
		   (add-grewplet-server-listener host port pw RGC
		      (lambda R #t)))) ;;this last parameter is a filter
	    (RGC 'send 'rollcall)
	    (.setSelectedComponent _JT pane)
	    (.pack _mainwin)
	    (.show _mainwin))
	  (if gs (gs 'quit)))))


  (define the-panel
    (letrec 
      ((connect-action
	(lambda (server?)
	  (action (lambda(e)
	   (.start (java.lang.Thread. (lambda()
            (let* ((user  (readstring (tag "auser")))
		   (group (readstring (tag "agroup")))
		   (host  (readstring (tag "host")))
		   (port  (readexpr   (tag "port")))
		   (pw    (readstring (tag "pw")))		      
		   (win   (show-alert {Trying to connect:<br>
				   user '[user]' to group '[group]'<br>@<br>
				   host: [host]:[port]})))
             (jsint.Procedure.catching
	      (lambda()
	        (let ((ugroup (if (and server? connected-to-rendezvous-server?)
				  (unique group sHost sPort "locale-names\n"))))
		 (grewp-init server? user
			     (if (and server? ugroup) ugroup group)
			     host port pw)
		 (if (and server? ugroup)
		     (SL:send-line sHost sPort
		      {add-locale\n("[ugroup]" "[host]" [port] "[pw]")\n}))
		 (.hide win))
               ) ;;close first (lamba() inside .catching
	      (lambda(e) (.hide win) (display {ERROR:[e]})(newline)
                        (U:error {[{<html>Connect/Start Failed!}
				   {<br>[(.getMessage e)]<br></html>}]}
				 network-win)))))))))))

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       (the-menu (lambda()
	   (border (north 
	      (menubar 
	       (menu "File"
		  (menuitem "Exit"
		     (action (lambda(e) 
                        (.hide network-win)
			(java.lang.System.exit 0)))))

	       (menu "Options"
		  (menuitem "Advanced Login"
  		     (action (lambda(e) (show-advanced))))
		  (menuitem "Rendezvous Server"
  		     (action (lambda(e) (show-rendezvous))))
		  (menuitem "Set Local Host"
		     (action (lambda(e)
			(let 
			  ((host (javax.swing.JOptionPane.showInputDialog
				  network-win
				  {Please enter local host address:} 
				  local-ip)))
			  (if (not (equal? #null host))
			      (set! local-ip host))
			  (writestring (tag "host") local-ip)
			  (writestring (tag "RSurl") local-ip))))))

	       (menu "Help"
		  (menuitem "GrewpEdit Tutorial"
  		     (action (lambda(e) (GH:show-help)))
		     (lambda(this) (.setEnabled this #f)))
		  (menuitem "About"
		     (action (lambda(e) (GH:show-about))))))))))

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       (advanced-options
	(border 
	 (north (the-menu))
	 (center
	  (col 'horizontal 'north
	       (row 'none 'center (label "Advanced Login" 
				    (HelveticaBold 18)))
	       (label " ") ;;spacer
	       (table 4 2
		 (label "Username") 
		 (tag "auser" (textfield "guest" 35))
		 (label "Groupname") 
		 (tag "agroup" (textfield "MyGroup" 35))
		 (label "Host") 
		 (tag "host" (textfield local-ip 35))
		 (label "Port") 
		 (tag "port" (textfield "14410" 35))
		 (label "Passphrase") 
		 (tag "pw" (textfield PW 35)))
	       (label " ") ;;spacer
	       (col 'none 'center
		 (row 'none 'center
		    (tag "start" (button "Start a Group"
					(connect-action #t)))
		    (tag "connect" (button "Connect to a Group"
					  (connect-action #f))))
		 (tag "close" (button "Close" (action (lambda (e)
			(.removeTabAt JT (.indexOfTab JT "Advanced Options"))
			(.setSelectedIndex JT 0)
			(.pack network-win)))))
	       ))))) ;;close col/col/center/border/advanced-options

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       
       (show-advanced 
	(lambda()
	  (if (= -1 (.indexOfTab JT "Advanced Options"))
	      (.add JT "Advanced Options" advanced-options))
	  (.setSelectedIndex JT (.indexOfTab JT "Advanced Options"))
	  (.pack network-win)))

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       (rendezvous-server-options
	(border 
	 (north (the-menu))
	 (center
	  (col 'horizontal 'north
	     (row 'none 'center (label "Rendezvous Server" 
					 (HelveticaBold 18)))
	     (table 4 2
		 (label "Default Host")
		 (tag "Durl" (textfield sHost 35
				(lambda(this) (.setEnabled this #f))))
		 (label "Default Port")
		 (tag "Dport" (textfield {[sPort]} 35
				(lambda(this) (.setEnabled this #f))))
		 (label "Host") 
		 (tag "RSurl" (textfield local-ip 35))
		 (label "Port") 
		 (tag "RSport" (textfield "14400" 35)))
	     (col 'none 'center
		(row 'none 'center
		     (tag "RSstart" (button "Start my Server" 
		        (action(lambda(e) 
			   (.start (java.lang.Thread. (lambda()
				 (rendezvous-server-start
				  (readexpr (tag "RSport")))
				 (.doClick (tag "RSclose")))))))))
		     (tag "RSconnect" (button "Connect to a Server"
                        (action(lambda(e) 
			   (.start (java.lang.Thread. (lambda()
				 (rendezvous-server-connect
				  (readstring (tag "RSurl"))
				  (readexpr (tag "RSport")))
				 (.doClick (tag "RSclose")))))))))
		     (tag "Dconnect" (button "Connect to default Server"
                        (action(lambda(e) 
			   (.start (java.lang.Thread. (lambda()
				 (rendezvous-server-connect
				  (readstring (tag "Durl"))
				  (readexpr (tag "Dport")))
				 (.doClick (tag "RSclose"))))))))))
		(tag "RSclose" (button "Close" 
		   (action (lambda (e)
		       (.removeTabAt JT (.indexOfTab JT "Rendezvous Options"))
		       (.setSelectedIndex JT 0)
		       (.pack network-win)))))
	       )))))

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


       (show-rendezvous
	    (lambda()
	      (if (= -1 (.indexOfTab JT "Rendezvous Options"))
		  (.add JT "Rendezvous Options" rendezvous-server-options))
	      (.setSelectedIndex JT (.indexOfTab JT "Rendezvous Options"))
	      (.pack network-win)))

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;        (fix-view (lambda()
; 	  (tag "first-time?" #f)
; ; 	  (.setFixedCellWidth (tag "groups") 
; ; 			      (.getWidth (tag "groups")))
; ; 	  (.removeTabAt JT (.indexOfTab JT "Login"))
; ; 	  (.insertTab JT "Login" #null PANE #null 0)
; ; 	  (.setSelectedIndex JT 0)
; 	  (.pack network-win)))

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       (go-rendezvous
	(lambda ()
	  (show-advanced)
	  (let* ((user (readstring (tag "user")))
		 (group (readstring (tag "group")))
		 (valid? (valid-locale? group))
		 (locale (if valid?
			     (SL:send-line sHost sPort {locale\n[group]\n})
			     (list local-ip (get-port) ;;14444
				   "public")))) ;(U:get-port)))))
	    (.setSelectedValue (tag "groups") group #t)
	    (writestring (tag "auser") user)
	    (writestring (tag "agroup") group)
	    (writestring (tag "host") (first locale))
	    (writestring (tag "port") (second locale))
	    (if valid?
		(let ((pw (if (equal? PW (third locale))
			      PW
			      (javax.swing.JOptionPane.showInputDialog
			       network-win
			       {Enter a passphrase for the group:}))))
		  (writestring (tag "pw") pw)
		  (.doClick (tag "connect")))
		(let ((pw (javax.swing.JOptionPane.showInputDialog
			   network-win 
			   {[{<html>Enter a passphrase to make this a private}
			     {<br>group or make it public using '[PW]'.</html>}]}
			   PW)))
		  (writestring (tag "pw") (if (string? pw) pw PW))
		  (.doClick (tag "start"))))
	    (.doClick (tag "close"))
	    (.setSelectedIndex JT 0)
; 	    (if (tag "first-time?")
; 		(fix-view))
       ))) ;;close let*/lambda/rendezvous

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       (PANE
	(border 
	 (north (the-menu))
	 (center
	  (col
	     (border
	      (north (col (row 'none 'center (label "Welcome To GrewpEdit" 
						    (HelveticaBold 18)))))
	      (south 
	       (row 'both 'center
		 (scrollpane (tag "status" 
		    (textarea 4 15 "Not Connected to a rendezvous Server!" 
			      (HelveticaBold 13))))))
	      (east
	       (table 2 1;; 'none 'center
		 (row 'none 'center 
		      (label "               Existing Groups               "
					   (HelveticaBold 14)))
		 (tag "sp" (scrollpane (tag "groups" 
		      (jlist (lambda(x)
			 (writestring (tag "group") x)
			 (if connected-to-rendezvous-server?
			     (let* ((locale (SL:send-line sHost sPort 
							  {locale\n[x]\n}))
				    (pw (if (equal? PW (third locale))
					    "public" "private"))
				    (host (first locale))
				    (port (second locale))
				    (users (fourth locale)))
			       (writestring (tag "status")
				 {[{Connected to rendezvous server @ }	
			   {[sHost]:[sPort]\n}
				   {[x] ==> Host: [host], Port: [port], }
				   {Users: [users], Passphrase: [pw]}]}))
			     (writestring (tag "status")
			       "Not Connected to a rendezvous Server!"))
			 (.pack network-win)
	      )))))))
	      (center
	       (col 
		(label " ") ;;spacer
		(row 'horizontal 'center
		    (table 4 2
		       (label "") (label "")
		       (label "Username")
		       (tag "user" (textfield "guest" 15))
		       (label "Groupname")
		       (tag "group" (textfield "NewGroup" 15
			      (action(lambda(e) (go-rendezvous)))))
		       (label "")
		       (row 'none 'east (button "GO" 
			  (action(lambda(e) (go-rendezvous)))))))
		(label " ") ;;spacer
	      ))) ;;close col/center/border
	))));;close col/center/border/PANE

       ) ;;close let* definitions

      PANE
  )) ;;close letrec/the-panel

  (.addChangeListener JT
      (jsint.Listener. (lambda(e)
	   (.pack network-win))))

  (.start (java.lang.Thread. (lambda()
     (rendezvous-server-connect sHost sPort))))

;   (.setVerticalScrollBarPolicy (tag "sp")
;     javax.swing.JScrollPane.VERTICAL_SCROLLBAR_ALWAYS$)
;   (.setHorizontalScrollBarPolicy (tag "sp")
;     javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS$)

;  (tag "first-time?" #t)
  (.start (java.lang.Thread. (lambda()
     (let loop ()
       (java.lang.Thread.sleep 1500L)
       (if (.isVisible the-panel)
	   (local-check)
	   (loop))))))
  the-panel
)