;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
{
 GrewpDocument.scm

 This module defines
   (make-document-panel G usercolor grewpname pw mainwin JT)
 where G is a GroupClient, mainwin and JT are the window and tabbed pane
   in which this panel will sit (this should be abstracted away....)

 The procedure creates a GroupTextAreaClient (GTAC) panel with a Chat
 panel that is connected to the GroupTextAreaServer (GTAS) running on 
 the same host and port as G. It also creates some File/Edit menus and 
 an awareness widget

 Note: tracking mode does jump to correct trackee position but from
   then on it only follows trackee inserts and deletes and not caret
   movement. Thus, user A tracking user B will stay at the last edit
   location of user B and will not follow all the movement of B if B
   is tracking C because B is not making edits but is moving with C.

   ;;Also if a person is being tracked at the end of a document a flury
   ;;of bad position errors occur for both tracker and trackee
}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(environment-import "groupscheme/platform/util/FileUtil.scm" "FileU:")
(environment-import "groupscheme/platform/util/Menu.scm" "Menu:")
(environment-import "groupscheme/platform/util/CodingEnv.scm" "ce:")
(environment-import "groupscheme/platform/util/Util.scm" "U:")
(environment-import "groupscheme/platform/util/Bindings.scm" "Bindings:")
(environment-import "groupscheme/platform/util/bindings/grewp.scm" "grewp:")
(environment-import "groupscheme/platform/GS/GroupClient.scm" "GC:")
(environment-import "groupscheme/platform/GS/GroupChat.scm" "GChat:")
(environment-import "groupscheme/platform/GS/GroupTextAreaClient.scm" "GTAC:")
(environment-import "groupscheme/platform/GS/GroupTextAreaServer.scm" "GTAS:")
(environment-import "groupscheme/platform/grewplet/MemberList.scm" "ML:")
(environment-import "jlib/Swing.scm")

;; usercolor initializes GTAC but then it can change so
;; it should not be used after that... need to change any
;; usage besides the init to (GTAC 'get 'color)
(define (make-document-panel RGC usercolor grewpname pw mainwin JT)
  (define user (RGC 'get 'name))
  (define host (first (RGC 'get 'address)))
  (define port (second (RGC 'get 'address)))
  (define G (GC:make-group-client user grewpname host port pw))
  (define GTAC (GTAC:make-GroupTextAreaClient G usercolor 5 50))
  (define GChat (GChat:make-p2p-group-chat 'name user 'color usercolor 
					   'mode 'gui))
  (define bindings (Bindings:make-bindings))

  (define RESULTS (textarea 5 50 usercolor))
  (define remote-result-hash (Hashtable.))
  (define remote-result-action
    (lambda R (G 'send 'code-env-results 'update
		 user (GTAC 'get 'colorRGB) R)))

  (define filehandler (FileU:makeFileHandler mainwin (GTAC 'gui)))
  (define save-request-action (lambda() (filehandler 'careful-save)))
  (define code-env (ce:makeCodingEnv (GTAC 'gui) RESULTS remote-result-action
				     save-request-action))
  ;; load the file into the code-env if they open or save-as file
  (define x (filehandler 'set-procedure (lambda (file) (code-env 'load file))))

  (define (remote-results-update)
    (define (join u)
      (if (and (not (equal? u user)) (not (equal? u "system")))
	  (let ((ta (textarea 5 50)))
	    (.put remote-result-hash u ta)
	    (.setEditable ta #f)
	    (G 'send 'code-env-results u user
	       (GTAC 'get 'colorRGB) (list '.setText (.getText RESULTS)))
	    (.addTab (code-env 'output-pane) u (scrollpane ta)))))

    (define (leave u)
      (if (and (not (equal? u user)) (not (equal? u "system")))
	  (let ((index (.indexOfTab (code-env 'output-pane) u)))
	    (if (.containsKey remote-result-hash u)
		(.remove remote-result-hash u))
	    (if (> index -1)
		(.remove (code-env 'output-pane) index)))))

    (define members (G 'get 'users-in-my-group))
    (define keys (array->list (.toArray (.keySet remote-result-hash))))

    ;; if in the group but not in the key set then join
    (for-each (lambda(m) (if (not (member m keys)) (join m))) members)
    ;; update keys variable
    (set! keys (array->list (.toArray (.keySet remote-result-hash))))
    ;; if in the key set but not in the group then leave
    (for-each (lambda(k) (if (not (member k members)) (leave k))) keys))


  (define (add-remote-result-listeners)
;;;;;;;;;;;; REALLY STRANGE:
;;;;;;;;;;;; Swing.scm is loaded but still
;;;;;;;;;;;; can't do (eval 'writestring)
;;;;;;;;;;;; but can do (apply writestring ...)
;;;;;;;;;;;; but can (eval '.setText)
;;;;;;;;;;;; is eval not from the loaded environments?
;;;;;;;;;;;; thus the need for the hack
    (define (hack-proc sym)
      (case sym
	((.append appendstring) .append)
	((.setText writestring) .setText)
	((.setEditable) .setEditable)))
    ;;This listener updates the RESULTS of other members
    (G 'add-listener 'code-env-results
       (lambda (cmd handle _user _color update)
	 (let ((reflect (lambda ()
		  (let ((ta (.get remote-result-hash _user))
			(proc (hack-proc (first update))));;(eval (first update))))
		    (if (and (.isInstance javax.swing.JTextArea.class ta)
			     (procedure? proc))
			(begin 	  ;; remove alpha from color
			  (.setBackground ta (apply color
				 (reverse (list-tail (reverse _color) 1))))
			  (apply (eval proc) (cons ta (rest update)))) )))))

	   (case handle
	     ;; send result update to the requesting user directly
	     ((request)
	         (remote-results-update)
		 (if (not (equal? user _user)) (reflect))
		 (G 'send 'code-env-results _user user
		    (GTAC 'get 'colorRGB) (list '.setText (.getText RESULTS))))
	     ;; a normal [non directed] update
	     ((update)  ;;SHOULD use id for authentification
	         (if (not (equal? user _user)) (reflect)))
	     ;; an update only for user [user == handle == (G 'get 'name)]
	     (else 
	         (if (and (equal? handle user) (not (equal? user _user)))
		     (begin (remote-results-update) (reflect))))) )
    ))

    ;;listen for all incoming and outgoing users
    (G 'add-listener 'GROUPSERVER
       (lambda (cmd . R)
	 (case (first R)
	   ((newuser userleft)
	    (remote-results-update)))))

  ) ;; close result listener definitions



;   (define filehandler (FileU:makeFileHandler mainwin (GTAC 'gui)
; 			(lambda (file) (code-env 'load file))))

  (define member-panel
    (letrec 
	(;; "trackmark" is the variable that stores your location
	 ;; before you enter a tracking mode.
	 (trackmark #f)
	 ;; "end-tracking" is a listener used to break out of tracking
	 (end-tracking ;; mode when a user starts typing
	  (jsint.Listener. (lambda(e)
	    (cond
	     ((= (.getID e) java.awt.event.KeyEvent.KEY_TYPED$) 
	      (java.awt.EventQueue.invokeLater
	       (tryCatch
		(lambda () (mark 'self) 
			(.setSelectedValue
			 ;; get the scrollpane from member-panel then
			 ;; get the viewport from the scrollpane then
			 ;; get the jlist from the viewport
			 (.getComponent (.getComponent (.getComponent 
			      member-panel 1) 0) 0)
			 user #t))
		(lambda(e) 
		  (display {\nend-tracking invokeLater Error: [e]\n})))))))))
	 ;; And "mark" is the procedure returned by this let and it sets the 
	 (mark ;; trackmark value or clears it.
	  (lambda (trackee)
	    (GTAC 'set 'tracking trackee)
	    (cond 
	     ((eq? trackee 'self)
	      (if trackmark (.removeKeyListener (GTAC 'gui) end-tracking))
	      (set! trackmark #f))
	     ((not trackmark)
	      (.addKeyListener (GTAC 'gui) end-tracking)
	      (set! trackmark (.createPosition (.getDocument (GTAC 'gui))
				    (.getCaretPosition (GTAC 'gui))))))))
	 ;; This is the jlist selection CallBack procedure that uses 
	 ;; "mark" and "update-tracking" listener to help facilitate
	 (callback ;; the tracking change made by a user
	  (lambda (member)
	    (cond 
	     ((equal? member "system")
	      (mark 'all)
	      (.requestFocusInWindow (GTAC 'gui)))
	     ((and (equal? member user) trackmark)
	      (.setCaretPosition (GTAC 'gui) (.getOffset trackmark))
	      (mark 'self)
	      (.requestFocusInWindow (GTAC 'gui)))
	     ((not (equal? member user))
	      (G 'send 'update-tracking 'request user member null null)))))
	);;letrec defines
     
      ;;This listener changes the tracking mode (i.e a user, all or self)
      ;;conveniently saving your location for a later return when not in
      ;;self-tracking mode
      (G 'add-listener 'update-tracking
	 (lambda (cmd handle asker _user id pos)
	   (case handle
	     ((request)
	      (if (equal? user _user)
		  (G 'send 'update-tracking
		     'response asker _user (G 'get 'id) 
		     (.getCaretPosition (GTAC 'gui)))))
	     ((response)
	      (if (equal? user asker)
		  (begin (mark id)
			 (.setCaretPosition (GTAC 'gui) pos)
			 (.requestFocusInWindow (GTAC 'gui))))))))

      (ML:make-member-list-panel G usercolor
        (lambda(c)
	  (GChat 'set 'color c)
	  (GTAC 'set 'color (U:colortransform c))
	  (.setBackground (GChat 'get 'chatarea) c)
	  ;; remove alpha from color
	  (.setBackground RESULTS (java.awt.Color. (.getRGB c)))
	  (G 'send 'code-env-results 'update user
	     (GTAC 'get 'colorRGB) (list '.append "")))
	callback
	(lambda(e) (.setSelectedIndex (.getSource e) -1)))))

  

  (define (make-menu TA)
    (let* 
	( (fmenu (menu "File"))
	  (emenu (menu "Edit"))
	  (bmenu (menu "Bindings"))
	  (fontmenu (menu "Font"))
	  (pmenu (javax.swing.JPopupMenu. "Popup"))
	  (close-action
	   (lambda()
	     (GTAC 'logout)
	     (G 'logout)
	     (RGC 'send 'rollcall)
	     (.remove JT (.getSelectedComponent JT))))
	  (addBindingsMenuItems 
	   (lambda (menu)
	     (let* ((addEmacsFileBindings
		     (lambda ()
		       (bindings 'add TA 'save-buffers-kill-emacs '((CTRL X) 
								    (CTRL C))
				 (lambda R (filehandler 'careful-save)
					   (close-action)))
		       (bindings 'add TA 'save-buffer '((CTRL X) (CTRL S))
				 (lambda R (filehandler 'careful-save)))
		       (bindings 'add TA 'write-file '((CTRL X) (CTRL W))
				 (lambda R (filehandler 'save-as)))
		       (bindings 'add TA 'kill-buffer '((CTRL X) (K))
				 (lambda R (filehandler 'new)))
		       (bindings 'add TA 'find-file '((CTRL X) (CTRL F))
				 (lambda R (filehandler 'open)))))
		    (addWindowsFileBindings
		     (lambda ()
		       (bindings 'add TA 'save-buffers-kill-emacs '(CTRL W)
				 (lambda R (filehandler 'careful-save)
					   (close-action)))
		       (bindings 'add TA 'save-buffer '(CTRL S)
				 (lambda R (filehandler 'careful-save)))
		       (bindings 'add TA 'kill-buffer '(CTRL N)
				 (lambda R (filehandler 'new)))
		       (bindings 'add TA 'find-file '(CTRL O)
				 (lambda R (filehandler 'open)))))
		   (bg (javax.swing.ButtonGroup.))
		   (on (javax.swing.JRadioButtonMenuItem.
			   (groupscheme.platform.wrapper.ActionWrapper.
			    "Limited Emacs"
			    (lambda R (bindings 'remove-all TA)
				      (bindings 'load grewp:emacs TA)
				      (addEmacsFileBindings)))))
		   (off (javax.swing.JRadioButtonMenuItem.
			 (groupscheme.platform.wrapper.ActionWrapper. "Off"
			  (lambda R (bindings 'remove-all TA)
				    (bindings 'load grewp:windows TA)
				    (addWindowsFileBindings))))))
	       (for-each (lambda(x) (.add menu x) (.add bg x))
			 (list on off))
	       (.setEnabled on #f)
	       (.doClick off)
	       (.setEnabled off #f))))
	  );;close let defs
      ;;File Menu
      (Menu:addFileMenuItems filehandler fmenu)
      (.add fmenu (menuitem "Save Chat"
         (action (lambda(e) (FileU:static-save (GChat 'get 'chatarea))))))
      (.addSeparator fmenu)
      (.add fmenu (menuitem "Close"
	 (action (lambda (e) (close-action)))
	 (lambda(this) (.setMnemonic this #'C'))))
      ;;Edit Menu
      (Menu:addEditMenuItems emenu TA)
      (.addSeparator emenu)
      (addBindingsMenuItems bmenu)
      (.add emenu bmenu)
      (.addSeparator emenu)
      (.add emenu (menuitem "Show Kill Ring"
			    (action (lambda(e) (GTAC 'show-kill-win)))))
      ;;Font Menu
      (Menu:addFontMenu fontmenu TA)
      ;;Popup Menu
      (Menu:addEditMenuItems pmenu TA)
      (Menu:addPopupMenu pmenu TA)

      (menubar fmenu emenu fontmenu
	(row 'none 'center 
	     (label {GroupTextAreaClient for [user] on [host]:[port]})))))


  (define (make-caret-tracker name ta)
    ;;The label that displays the location of the caret
    (let ((bar (label {----[name]:    L1--C1--0%       }
		      (lambda(this)
			(.setFont this (.deriveFont (.getFont this) 9.5f))))))
      (.addCaretListener ta 
	(jsint.Listener. (lambda(e)
          (let ((line (.getLineOfOffset ta (.getCaretPosition ta))))
	    (writestring bar ;;Track Caret Movement
	      (string-append
	       {----[name]:    }
	       {L[(+ 1 line)]--}
	       {C[(+ 1 (- (.getCaretPosition ta) 
			  (.getLineStartOffset ta line)))]--}
	       {[(.intValue 
		  (* 100 (/ (+ 1 line) 
			    (Double. {[(.getLineCount ta)]}))))]%}))))))
      (row 'none 'west bar (.getBackground bar))))


  (define emacs-panel
    (col (make-caret-tracker grewpname (GTAC 'gui))
	 (bindings 'get 'gui)))

  (define doc-panel
    (border
     (center (hsplit 0.5 (scrollpane (GTAC 'gui))
		     (code-env 'output-pane)))
     (south (col emacs-panel (code-env 'control-pane)))))



  (define the-panel
    (border
      (north (make-menu (GTAC 'gui)))
      (center (hsplit 0.2 member-panel
		      (vsplit 0.2 (GChat 'get 'panel)
			          doc-panel)))))


  (G 'add-socket-closure-hook (lambda R 
     (U:error {[{<html>}
		{Your connection between '[grewpname]' and the server<br>}
		{has died! You can save your document, but you can't edit<br>}
		{'[grewpname]' anymore.</html>}]}
	      the-panel)
     (.setBackground (GTAC 'gui) (java.awt.Color. 0 255 255))))


  ;; setup shared code-env results
  (add-remote-result-listeners) 
  (remote-results-update) ;; create output areas for other users
  (G 'send 'code-env-results 'request user
     (GTAC 'get 'colorRGB) (list '.setEditable #f))


  (U:grewp-chat-setup G GChat usercolor
     {[{*** Welcome to the Chat space for '[grewpname]'\n}
       {*** The shared document is directly below this chat panel.\n}]})

  (.setTabSize (GTAC 'gui) 3)
  the-panel
 )


(define (grewplet cmd . R)
  (case cmd
    ((name) "CodeDocument")
    ((gui) (apply make-document-panel R))
    ((start-server) ;;R ==> host port grewpname pw
     (apply GTAS:make-GroupTextAreaServer R))
    ((remote-server?) #t)))