;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
{
 GrewpDocument.scm

 This module defines
   (make-document-panel G usercolor grewpname pw mainwin JT)
 where G is a GroupClient, mainwin and JT are the window and tabbed pane
   in which this panel will sit (this should be abstracted away....)

 The procedure creates a GroupTextAreaClient (GTAC) panel with a Chat
 panel that is connected to the GroupTextAreaServer (GTAS) running on 
 the same host and port as G. It also creates some File/Edit menus and 
 an awareness widget

 Note: tracking mode does jump to correct trackee position but from
   then on it only follows trackee inserts and deletes and not caret
   movement. Thus, user A tracking user B will stay at the last edit
   location of user B and will not follow all the movement of B if B
   is tracking C because B is not making edits but is moving with C.

   ;;Also if a person is being tracked at the end of a document a flury
   ;;of bad position errors occur for both tracker and trackee
}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(environment-import "groupscheme/platform/util/FileUtil.scm" "FileU:")
(environment-import "groupscheme/platform/util/Menu.scm" "Menu:")
(environment-import "groupscheme/platform/util/Util.scm" "U:")
(environment-import "groupscheme/platform/util/Bindings.scm" "Bindings:")
(environment-import "groupscheme/platform/util/bindings/grewp.scm" "grewp:")
(environment-import "groupscheme/platform/GS/GroupClient.scm" "GC:")
(environment-import "groupscheme/platform/GS/GroupChat.scm" "GChat:")
(environment-import "groupscheme/platform/GS/GroupTextAreaClient.scm" "GTAC:")
(environment-import "groupscheme/platform/GS/GroupTextAreaServer.scm" "GTAS:")
(environment-import "groupscheme/platform/grewplet/MemberList.scm" "ML:")
(environment-import "jlib/Swing.scm")

(define (make-document-panel RGC usercolor grewpname pw mainwin JT)
  (define user (RGC 'get 'name))
  (define host (first (RGC 'get 'address)))
  (define port (second (RGC 'get 'address)))
  (define G (GC:make-group-client user grewpname host port pw))
  (define GTAC (GTAC:make-GroupTextAreaClient G usercolor 5 50))
  (define GChat (GChat:make-p2p-group-chat 'name user 'color usercolor 
					   'mode 'gui))
  (define bindings (Bindings:make-bindings))

  (define member-panel
    (letrec 
	(;; "trackmark" is the variable that stores your location
	 ;; before you enter a tracking mode.
	 (trackmark #f)
	 ;; "end-tracking" is a listener used to break out of tracking
	 (end-tracking ;; mode when a user starts typing
	  (jsint.Listener. (lambda(e)
	    (cond
	     ((= (.getID e) java.awt.event.KeyEvent.KEY_TYPED$) 
	      (java.awt.EventQueue.invokeLater
	       (tryCatch
		(lambda () (mark 'self) 
			(.setSelectedValue
			 ;; get the scrollpane from member-panel then
			 ;; get the viewport from the scrollpane then
			 ;; get the jlist from the viewport
			 (.getComponent (.getComponent (.getComponent 
			      member-panel 1) 0) 0)
			 user #t))
		(lambda(e) 
		  (display {\nend-tracking invokeLater Error: [e]\n})))))))))
	 ;; And "mark" is the procedure returned by this let and it sets the 
	 (mark ;; trackmark value or clears it.
	  (lambda (trackee)
	    (GTAC 'set 'tracking trackee)
	    (cond 
	     ((eq? trackee 'self)
	      (if trackmark (.removeKeyListener (GTAC 'gui) end-tracking))
	      (set! trackmark #f))
	     ((not trackmark)
	      (.addKeyListener (GTAC 'gui) end-tracking)
	      (set! trackmark (.createPosition (.getDocument (GTAC 'gui))
				    (.getCaretPosition (GTAC 'gui))))))))
	 ;; This is the jlist selection CallBack procedure that uses 
	 ;; "mark" and "update-tracking" listener to help facilitate
	 (callback ;; the tracking change made by a user
	  (lambda (member)
	    (cond 
	     ((equal? member "system")
	      (mark 'all)
	      (.requestFocusInWindow (GTAC 'gui)))
	     ((and (equal? member user) trackmark)
	      (.setCaretPosition (GTAC 'gui) (.getOffset trackmark))
	      (mark 'self)
	      (.requestFocusInWindow (GTAC 'gui)))
	     ((not (equal? member user))
	      (G 'send 'update-tracking 'request user member null null)))))
	);;letrec defines
     
      ;;This listener changes the tracking mode (i.e a user, all or self)
      ;;conveniently saving your location for a later return when not in
      ;;self-tracking mode
      (G 'add-listener 'update-tracking
	 (lambda (cmd handle asker _user id pos)
	   (case handle
	     ((request)
	      (if (equal? user _user)
		  (G 'send 'update-tracking
		     'response asker _user (G 'get 'id) 
		     (.getCaretPosition (GTAC 'gui)))))
	     ((response)
	      (if (equal? user asker)
		  (begin (mark id)
			 (.setCaretPosition (GTAC 'gui) pos)
			 (.requestFocusInWindow (GTAC 'gui))))))))

      (ML:make-member-list-panel G usercolor
        (lambda(c)
	  (.setBackground (GChat 'get 'chatarea) c)
	  (GChat 'set 'color c)
	  (GTAC 'set 'color (U:colortransform c)))
	callback
	(lambda(e) (.setSelectedIndex (.getSource e) -1)))))

  
  (define (make-menu parent TA)
    (let* ((fmenu (menu "File"))
	  (emenu (menu "Edit"))
	  (fontmenu (menu "Font"))
	  (bmenu (menu "Bindings"))
	  (pmenu (javax.swing.JPopupMenu. "Popup"))
	  (filehandler (FileU:makeFileHandler parent TA))
	  (close-action
	   (lambda()
	     (GTAC 'logout)
	     (G 'logout)
	     (RGC 'send 'rollcall)
	     (.remove JT (.getSelectedComponent JT))))
	  (addBindingsMenuItems 
	   (lambda (menu)
	     (let* ((addEmacsFileBindings
		     (lambda ()
		       (bindings 'add TA 'save-buffers-kill-emacs '((CTRL X) 
								    (CTRL C))
				 (lambda R (filehandler 'careful-save)
					   (close-action)))
		       (bindings 'add TA 'save-buffer '((CTRL X) (CTRL S))
				 (lambda R (filehandler 'careful-save)))
		       (bindings 'add TA 'write-file '((CTRL X) (CTRL W))
				 (lambda R (filehandler 'save-as)))
		       (bindings 'add TA 'kill-buffer '((CTRL X) (K))
				 (lambda R (filehandler 'new)))
		       (bindings 'add TA 'find-file '((CTRL X) (CTRL F))
				 (lambda R (filehandler 'open)))))
		    (addWindowsFileBindings
		     (lambda ()
		       (bindings 'add TA 'save-buffers-kill-emacs '(CTRL W)
				 (lambda R (filehandler 'careful-save)
					   (close-action)))
		       (bindings 'add TA 'save-buffer '(CTRL S)
				 (lambda R (filehandler 'careful-save)))
		       (bindings 'add TA 'kill-buffer '(CTRL N)
				 (lambda R (filehandler 'new)))
		       (bindings 'add TA 'find-file '(CTRL O)
				 (lambda R (filehandler 'open)))))
		   (bg (javax.swing.ButtonGroup.))
		   (on (javax.swing.JRadioButtonMenuItem.
			   (groupscheme.platform.wrapper.ActionWrapper.
			    "Limited Emacs"
			    (lambda R ;(bindings 'remove-all TA)
				      (bindings 'load grewp:emacs TA)
				      (addEmacsFileBindings)))))
		   (off (javax.swing.JRadioButtonMenuItem.
			 (groupscheme.platform.wrapper.ActionWrapper. "Off"
			  (lambda R ;(bindings 'remove-all TA)
				    (bindings 'load grewp:windows TA)
				    (addWindowsFileBindings))))))
	       (for-each (lambda(x) (.add menu x) (.add bg x))
			 (list on off))
	       (.setEnabled on #f)
	       (.doClick off)
	       (.setEnabled off #f))))
	  );;close let defs
      ;;File Menu
      (Menu:addFileMenuItems filehandler fmenu)
      (.add fmenu (menuitem "Save Chat"
         (action (lambda(e) (FileU:static-save (GChat 'get 'chatarea))))))
      (.addSeparator fmenu)
      (.add fmenu (menuitem "Close"
	 (action (lambda (e) (close-action)))
	 (lambda(this) (.setMnemonic this #'C'))))
      ;;Edit Menu
      (Menu:addEditMenuItems emenu TA)
      (.addSeparator emenu)
      (addBindingsMenuItems bmenu)
      (.add emenu bmenu)
      (.addSeparator emenu)
      (.add emenu (menuitem "Show Kill Ring"
			    (action (lambda(e) (GTAC 'show-kill-win)))))
      ;;Font Menu
      (Menu:addFontMenu fontmenu TA)
      ;;Popup Menu
      (Menu:addEditMenuItems pmenu TA)
      (Menu:addPopupMenu pmenu TA)

      (menubar fmenu emenu fontmenu
	(row 'none 'center 
	     (label {GroupTextAreaClient for [user] on [host]:[port]})))))


  (define (make-caret-tracker name ta)
    ;;The label that displays the location of the caret
    (let ((bar (label {----[name]:    L1--C1--0%       }
		      (lambda(this)
			(.setFont this (.deriveFont (.getFont this) 9.5f))))))
      (.addCaretListener ta 
	(jsint.Listener. (lambda(e)
          (let ((line (.getLineOfOffset ta (.getCaretPosition ta))))
	    (writestring bar ;;Track Caret Movement
	      (string-append
	       {----[name]:    }
	       {L[(+ 1 line)]--}
	       {C[(+ 1 (- (.getCaretPosition ta) 
			  (.getLineStartOffset ta line)))]--}
	       {[(.intValue 
		  (* 100 (/ (+ 1 line) 
			    (Double. {[(.getLineCount ta)]}))))]%}))))))
      (row 'none 'west bar (.getBackground bar))))


  (define emacs-panel
    (col (make-caret-tracker grewpname (GTAC 'gui))
	 (bindings 'get 'gui)))

  (define doc-panel
    (border
     (center (scrollpane (GTAC 'gui)))
     (south emacs-panel)))

  (define the-panel
    (border
      (north (make-menu mainwin (GTAC 'gui)))
      (center (hsplit 0.2 member-panel
		      (vsplit 0.2 (GChat 'get 'panel)
			          doc-panel)))))

  (G 'add-socket-closure-hook (lambda R 
     (U:error {[{<html>}
		{Your connection between '[grewpname]' and the server<br>}
		{has died! You can save your document, but you can't edit<br>}
		{'[grewpname]' anymore.</html>}]}
	      the-panel)
     (.setBackground (GTAC 'gui) (java.awt.Color. 220 220 220))))

  (U:grewp-chat-setup G GChat usercolor
     {[{*** Welcome to the Chat space for '[grewpname]'\n}
       {*** The shared document is directly below this chat panel.\n}]})

  (.setTabSize (GTAC 'gui) 3)
  the-panel
 )


(define (grewplet cmd . R)
  (case cmd
    ((name) "Document")
    ((gui) (apply make-document-panel R))
    ((start-server) ;;R ==> host port grewpname pw
     (apply GTAS:make-GroupTextAreaServer R))
    ((remote-server?) #t)))