;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
{
 GrewpWhiteboard.scm

 This module defines
    (make-whiteboard-panel RGC usercolor filename pw mainwin JT)
 where RGC is a GroupClient, mainwin and JT are the window and tabbed pane
 in which this panel will sit (this should be abstracted away....)
}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(environment-import "groupscheme/platform/util/FileUtil.scm" "FileU:")
(environment-import "groupscheme/platform/util/Menu.scm" "Menu:")
(environment-import "groupscheme/platform/util/Util.scm" "U:")
(environment-import "groupscheme/platform/GS/GroupClient.scm" "GC:")
(environment-import "groupscheme/platform/GS/GroupChat.scm" "GChat:")
(environment-import "groupscheme/platform/GS/Whiteboard.scm" "WB:")
(environment-import "groupscheme/platform/grewplet/MemberList.scm" "ML:")
(environment-import "jlib/Swing.scm")

(define (make-whiteboard-panel RGC usercolor grewpname pw mainwin JT)
  (define user (RGC 'get 'name))
  (define host (first (RGC 'get 'address)))
  (define port (second (RGC 'get 'address)))
  (define G (GC:make-group-client user grewpname host port pw))
  (define WB (WB:make-whiteboard G usercolor))
  (define GChat (GChat:make-p2p-group-chat 'name user 'color usercolor 
					   'mode 'gui))
  (define member-panel
    (ML:make-member-list-panel G usercolor
     (lambda(c)
       (.setBackground (GChat 'get 'chatarea) c)
       (GChat 'set 'color c)
       (WB 'set 'color c))
     (lambda(x) (display x))
     (lambda(e) (.setSelectedIndex (.getSource e) -1))))

  (define (make-menu parent TA)
    (let* ((fmenu (menu "File"))
	   (emenu (menu "Edit"))
	   (pmenu (javax.swing.JPopupMenu. "Popup"))
	   (fhandler (FileU:makeFileHandler parent TA))
	   (close-action
	    (lambda()
	      (G 'logout)
	      (RGC 'send 'rollcall)
	      (.remove JT (.getSelectedComponent JT))))
	  );;close let defs
      ;;File Menu
      (.add fmenu (menuitem "Save..." (action (lambda (e) (fhandler 'save-as)))
			    (lambda(this) (.setMnemonic this #'S'))))
      (.addSeparator fmenu)
      (.add fmenu (menuitem "Close" (action (lambda (e) (close-action)))
			    (lambda(this) (.setMnemonic this #'C'))))
      ;;Edit Menu
      (Menu:addEditMenuItems emenu TA)
      ;;Popup Menu
      (Menu:addEditMenuItems pmenu TA)
      (Menu:addPopupMenu pmenu TA)

      (menubar fmenu emenu
	(row 'none 'center 
	     (label {Grewp Chat for [user] on [host]:[port]})))))

  (define the-panel
    (border
      (north (make-menu mainwin (GChat 'get 'chatarea)))
      (center (hsplit 0.2 member-panel 
		      (vsplit 0.2 (GChat 'get 'panel)
			      (WB 'get 'gui))))))

  (G 'add-socket-closure-hook (lambda R 
     (U:error {Your connection between [grewpname] and the server has died!}
	      the-panel)))

  (U:grewp-chat-setup G GChat usercolor
     {[{*** Welcome to the Chat space for '[grewpname]'\n}
       {*** The whiteboard is directly below this chat panel.\n}]})

  the-panel
 )


(define (grewplet cmd . R)
  (case cmd
    ((name) "Whiteboard")
    ((gui) (apply make-whiteboard-panel R))))