(environment-import "jlib/Swing.scm")
(environment-import "elf/basic.scm")
(environment-import "groupscheme/platform/util/bindings/defaults.scm" "def:")


;;;;;;;;;;;;;;;;;;;;;;;;;;HELPER PROCEDURES;;;;;;;;;;;;;;;;;;;;;;
(define (hash-values H) (array->list (.toArray (.values H))))
(define (hash-keys H) (array->list (.toArray (.keySet H))))
(define (toStringQ S)  (jscheme.REPL.printToString S #t))
;; This is like map except that if the procedure being mapped evaluates
;; to #f or () then x it is not included in the result list
(define (map-strip proc L)
  (let ((newL '()))
    (map (lambda (x)
	   (if ((eval proc) x)
	       (set! newL (cons x newL))))
	 L)
    newL))
;; Expects a list of lists "L", some object "x" and a position "pos".
;; Then returns a list of all the interior lists "iList" that have
;; the "x" at "pos".
;; Ex: (assoc-ref '((a b c) (c d) (e b)) 'b 1)  ==> '((e b) (a b c))
(define (assoc-ref L x pos)
  (map-strip (lambda (iList) ;;interior list
	       (if (and (list? iList) (> (length iList) pos))
		   (if (equal? x (list-ref iList pos))
		       iList)))
	     L))

;;A procedures to retrieve and store objects in the hashtables
(define (namer hash)
  (lambda (name . obj)
    (if (null? obj)
	(.get hash name)
	(begin (.put hash name (first obj)) (first obj)))))
;;;;;;;;;;;;;;;;;;;;;;;;;;END OF HELPERS;;;;;;;;;;;;;;;;;;;;;;

(define (make-bindings)
  (define GLOBALS (Hashtable.))
  (define tag (namer GLOBALS))
  (define DEFAULTS (Hashtable.))
  (define def (namer DEFAULTS))
  (define COMMANDS (Hashtable.))
  ;;(define cmd (namer COMMANDS))
  (define BINDINGS (Hashtable.))
  (define cmd (namer BINDINGS))

  ;; returns: the KeyStroke represented by ksl = KeyStrokeList 
  ;;          ex: ksl => (modifiers code) => (CTRL ALT K)
  ;; mask: is the sum of the modifiers:
  ;;         * java.awt.event.KeyEvent.SHIFT_MASK (1)
  ;;         * java.awt.event.KeyEvent.CTRL_MASK (2)
  ;;         * java.awt.event.KeyEvent.META_MASK (4)
  ;;         * java.awt.event.KeyEvent.ALT_MASK (8) 
  ;; symbol-expr: reconstructs an expression that was broken into
  ;;              a list of symbols and evaluates it
  (define (get-KeyStroke ksl) 
    (define (symbol-expr symbols)
      (eval (string->symbol (apply string-append symbols))))
    (let* ((modifiers (rest (reverse ksl)))
	   (code (first (reverse ksl)))
	   (event 'java.awt.event.KeyEvent.)
	   (mask
	    (cond ((null? modifiers) 0)
		  ((number? (first modifiers)) (first modifiers))
		  (else (apply + (map symbol-expr
				      (map (lambda(M) `(,event ,M _MASK$))
					   modifiers))))))
	   (keycode (symbol-expr `(,event VK_ ,code $))))
      (javax.swing.KeyStroke.getKeyStroke keycode mask #f))) ;;#f = on-release?



  ;; KeyEvent Listener that creates keystrokes, from non-modifier
  ;; key pressed events then processes it
  ;; Event types: PRESSED || RELEASED || TYPED
  (define KSListener
    (jsint.Listener. (lambda(e)
     (.consume e)
     (cond
      ((= (.getID e) java.awt.event.KeyEvent.KEY_PRESSED$) 
       (if (and (!= (.getKeyCode e) java.awt.event.KeyEvent.VK_SHIFT$)
		(!= (.getKeyCode e) java.awt.event.KeyEvent.VK_ALT$)
		(!= (.getKeyCode e) java.awt.event.KeyEvent.VK_CONTROL$)
		(!= (.getKeyCode e) java.awt.event.KeyEvent.VK_META$))
	   (invoke (lambda ()
	     (apply (process (javax.swing.KeyStroke.getKeyStroke
			      (.getKeyCode e) (.getModifiers e))
			     (tag 'owner) #t))))))))))

  ;; exit-buffer if the owner of this listener regains the focus
  (define FocusListener
    (jsint.Listener. (lambda(e)
     (if (= (.getID e) java.awt.event.FocusEvent.FOCUS_GAINED$)
	 (exit-buffer)))))


  (define (enter-buffer)
    (invoke(lambda ()
	(.setEnabled (tag 'buffer) #t)
	(if (not (.requestFocusInWindow (tag 'buffer)))
	    ;;should also exit command mode here
	    (.clearGlobalFocusOwner
	     (java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager)))
	(writestring (tag 'buffer) "")
	(.addKeyListener (tag 'buffer) KSListener)
	(.addFocusListener (tag 'owner) FocusListener))))

  (define (exit-buffer)
    (invoke (lambda()
	(.removeFocusListener (tag 'owner) FocusListener)
	(.removeKeyListener (tag 'buffer) KSListener)
	(writestring (tag 'status) "")
	(writestring (tag 'buffer) "")
	(.setEnabled (tag 'buffer) #f)
	(if (not (.requestFocusInWindow (tag 'owner)))
	    (.clearGlobalFocusOwner
	     (java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager)))
	(tag 'prefix ())
	(tag 'outcomes ()))))

  (define (do-cmd key)
    (invoke (lambda ()
	(apply (cmd key)))))
  
  (define (match-prefix)
    (cond ((null? (tag 'outcomes))                   ;;no match
	   (exit-buffer))
	  ((member (tag 'prefix) (tag 'outcomes))    ;;perfect match
	   (let ((prefix (tag 'prefix)))
	     (exit-buffer)
	     (do-cmd prefix)))	   
	  (else                                      ;;still pending
	   (writestring (tag 'status) (toStringQ (tag 'prefix))))))

  (define (process KS C prefix?)
    (lambda R (invoke (lambda()
	 (cond 
	   ((not prefix?) (do-cmd KS))
	   ((null? (tag 'prefix))
	    (tag 'owner C)
	    (enter-buffer)
	    (tag 'outcomes (assoc-ref (map-strip list? (hash-keys BINDINGS)) 
				      KS 0))
	    (tag 'prefix (list KS))
	    (match-prefix))
	   (else
	    (tag 'outcomes (assoc-ref (tag 'outcomes) 
				      KS
				      (length (tag 'prefix))))
	    (tag 'prefix (append (tag 'prefix) (list KS)))
	    (match-prefix)))))))


  ;; This procedure parameters are
  ;;    Component, Name, KeyStrokeList, Action, and option Description
  ;; *KeyStrokeList* can either be a list with keystroke specs:
  ;;    (CTRL SHIFT 5)  ==> non-prefix KeyStrokeList
  ;; or a list of those spec lists:
  ;;    ((CTRL X) (CTRL Y) (ALT SHIFT 2))  ==> prefix KeyStrokeList
  ;; The input map and action map of *Component* is used to store an
  ;; action wrapped procedure that processes the keystroke genereted
  ;; by *KSL* or the first keystroke of the keystroke sequence (a list
  ;; of keystrokes) generated by a prefix *KSL*.
  ;; The *Action* being added as *Name* can be passed in as a procedure
  ;; or a string. If *A* is a string then it is suppose to be a java
  ;; default action name and is converted to a procedure. If *A* is a 
  ;; symbol then it is suppose to be a scheme defined action name and 
  ;; is set to the procedure stored for that name. Then *A* is stored in
  ;; BINDINGS with a key of either KSL's keystorke or keystroke sequence.
  (define (add C N KSL A . D) 
    (let* ((input-map (.getInputMap C))
	   (action-map (.getActionMap C))
	   (prefix? (list? (first KSL)))
	   (ks (get-KeyStroke (if prefix? (first KSL) KSL)))
	   (ksequence (if prefix? (map get-KeyStroke KSL)))
	   (name (if prefix? {Prefix[(.toString ks)]} (symbol->string N)))
	   (jam {[A]})) ;; a defualt java action-map name
      (if (string? A)
	  (set! A (lambda R
		    (.actionPerformed
		     (.get action-map jam)
		     (java.awt.event.ActionEvent. 
		      C java.awt.event.ActionEvent.ACTION_PERFORMED$ jam)))))
      (if (symbol? A)
	  (set! A (apply (def A) `(,handler ,C))))
      (.put input-map ks name)
      (.put action-map name (groupscheme.platform.wrapper.ActionWrapper. 
			     name (process ks C prefix?)))
      (cmd (if prefix? ksequence ks) A)
      ;;this is like a bindings info table
      ;;(bind N `(,N ,KSL ,A ,(first D)))))
      (cmd N `(,KSL ,A ,(first D)))))


  ;; This procedure takes a Component and a Name. It removes binding
  ;; Name from BINDINGS. Then if Name corresponds to a non-prefix 
  ;; Command or if it is the only binding with this prefix then it
  ;; is also removed from the Component's input-map and action-map.
  (define (remove C N)
    (let ((binding (.remove BINDINGS N)))
      (if (not (equal? #null binding))
	  (let* ((input-map (.getInputMap C))
		 (action-map (.getActionMap C))
		 (KSL (first binding))
		 (prefix? (list? (first KSL)))
		 (ks (get-KeyStroke (if prefix? (first KSL) KSL)))
		 (ksequence (if prefix? (map get-KeyStroke KSL)))
		 (name (if prefix? {Prefix[(.toString ks)]}
			           (symbol->string N))))
	    (.remove BINDINGS (if prefix? ksequence ks))
	    ;;(.remove COMMANDS (if prefix? ksequence ks))
	    (if (or (not prefix?)
		    (null? (assoc-ref (map-strip list? (hash-keys BINDINGS))
				      ks 0)))
		(.put input-map ks #null)
		(.put action-map name #null)))
	  (display {[N] does not exist!}))))

  (define handler
    (lambda (command . args)
	(case command
	  ((add) (apply add args))
	  ((remove) (remove (first args) (second args)))
	  ((remove-all) (map (lambda(x) (remove (first args) x))
			     (hash-keys BINDINGS)))
	  ((tag) (apply tag args))
	  ((get)
	   (case (first args)
	     ((list)
	      (map-strip list?
		(map (lambda (key)
		       (if (symbol? key)
			   `(,key ,(toStringQ (first (cmd key))))))
		     (hash-keys BINDINGS))))
	     ((supported)
	      (append (map (lambda(x) {'[(first x)]}) def:defaults)
		      (map (lambda(x) {"[x]"}) def:java-defaults)))
	     ((gui)
	      (tag 'gui (splitrow 1 30 
			    (row 'none 'center white (tag 'status))
			    (row 'horizontal 'west white (tag 'buffer)))))))
	  ((load) ;; args ==> (bindings-list TC)
	   (handler 'remove-all (first args))
	   (map (lambda(x) (apply add (append (list (second args)) x)))
		(first args)))
	  (else #t))))


  ;; A list containing prefix keystrokes to a keystroke sequence command
  (tag 'prefix '()) ;;ex: '(ctrl-x ctrl-y)
  ;; A list keystroke sequence commands that begin with (tag 'prefix)
  (tag 'outcomes '()) ;ex: '((ctrl-x ctrl-y ctrl-z) (ctrl-x ctrl-y k))
  ;; This will become the component that triggers a call of (enter-buffer)
  (tag 'owner '())
  ;;used to get input from the user
  (tag 'buffer (textarea 1 15 
      (lambda(this) (.setFont this (.deriveFont (.getFont this) 9.5f)))))
  (.setEnabled (tag 'buffer) #f)
  (tag 'status (label ""
      (lambda(this) (.setFont this (.deriveFont (.getFont this) 9.5f)))))
  ;; load default actions
  (map (lambda(x) (def (first x) (second x))) def:defaults)

  handler
)



(define (invoke lambda-expr)
  (java.awt.EventQueue.invokeLater
   (tryCatch 
    lambda-expr
    (lambda(e) 
      (javax.swing.JOptionPane.showMessageDialog
       #null {Bindings Invoke Error: [e]} "Bindings"
       javax.swing.JOptionPane.ERROR_MESSAGE$)))))


;;;;;;;;;;;;;;;;;;;;;;;;;DEMO STUFF;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (demo)
  (define cc (make-bindings))
  (define tc (textarea 10 10))
  (define emacs-panel (cc 'get 'gui))
  (define win 
    (window "Bindings Test"
	    (border (north (label "Control-H For Help"))
		    (center (scrollpane tc))
		    (south emacs-panel)
		    (west (col
 (button "remove" (action(lambda(e) (cc 'remove-all tc))))
(button "load" (action(lambda(e) (cc 'load config tc))))))
)))
  (.pack win)
  (.show win)
  (cc 'load config tc))

(define config
  (list
   ;; name                KeyStroke                 Action
   ;; ---                 ---------                 ------
  `(goto-line            (META L)                  goto-line)
   `(show-help            (CTRL H)                  show-help)
   `(undo                 (CTRL SHIFT MINUS)        emacs-undo)
   `(kill-line            (CTRL K)                  kill-line)
   `(kill-region          (META W)                  kill-region)
   `(yank                 (CTRL Y)                  yank)
   `(query-replace        ((ESCAPE) (SHIFT 5))      query-replace)
   `(isearch              (CTRL S)                  isearch)
   `(isearch-backward     (CTRL R)                  isearch-backward)
   `(beginning-of-line    (CTRL A)                  ,javax.swing.text.DefaultEditorKit.beginLineAction$)
   `(end-of-line          (CTRL E)                  ,javax.swing.text.DefaultEditorKit.endLineAction$)
  ) ;;close list
) ;;close define

;(demo)
;(temp (ALT H) (lambda R (display "its working")))