;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Coding Environment
;; * an edit/watch choice component
;; * an Java evaluation button
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(environment-import "groupscheme/platform/util/HTMLPane.scm" "HP:")
(environment-import "using/run.scm")
(language-import "using/run.scm")
(environment-import "jlib/Swing.scm")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;; Helper Procedures
(define (time)
  (let ((df (java.text.DateFormat.getTimeInstance)))
    (.format df (java.util.Date.))))

(define (make-path L)
  (let loop ((p L)
	     (s "."))
    (if (null? p)
	s
	(loop (cdr p) {[s][($ "path.separator")][(car p)]}))))

(define (string->strlist str)
  (let ((tokenizer (java.util.StringTokenizer. str " " #f))
	 (convert (lambda(tx)
		    (let loop ((L '()))
		      (if (.hasMoreTokens tx)
			  (loop (append L `(,(.nextToken tx))))
			  L)))))
    (convert tokenizer)))

(define (list->spacedstr L)
  (apply string-append (map (lambda(x) {[(.toString x)] })
			    (flatten L))))

(define (filechoose fc parent type . R)
  (let ((option (first R)))
    (if (= (.showOpenDialog fc parent)
	   javax.swing.JFileChooser.APPROVE_OPTION$)
	(case type
	  ((Add)
	   (append
	    (if (list? option)
		option
		(list option))
	    (if (.isMultiSelectionEnabled fc)
		(array->list (.getSelectedFiles fc))
		(.getSelectedFile fc))))
	  ((Browse)
	   (if (.isMultiSelectionEnabled fc)
	       (array->list (.getSelectedFiles fc))
	       (.getSelectedFile fc))))
	option)))

(define (err parent str)
  (javax.swing.JOptionPane.showMessageDialog
   parent str "GrewpEdit"
   javax.swing.JOptionPane.ERROR_MESSAGE$))
;;;;;;;;;;; End of Helper Procedures
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;; R ==> (resultTA, reflect-results-procedure, save-request-procedure)
(define (makeCodingEnv TA . R)

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; The important variables
  (define subprocess #null)
  (define command-thread (java.lang.Thread. (lambda() #t)))
  (define subprocess-thread (java.lang.Thread. (lambda() #t)))
  (define drain-err-thread (java.lang.Thread. (lambda() #t)))
  (define drain-in-thread (java.lang.Thread. (lambda() #t)))

  (define drain-process? #f)

  (define sourcefile null)
  (define workingdir null)

  (define compilefiles '())
  (define classpathjc '())
  (define sourcepath '())
  (define directory '())

  (define mainclass '())
  (define classpathj '())
  (define appleturl '())

  (define java #f)
  (define javac #f)
  (define custompro #f)
  (define customproc #f)
  (define c++pro #f)
  (define c++proc #f)
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;STAR OF GUI;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  (define checkbox (lambda R
    (let ((C (javax.swing.JCheckBox. (first R))))
      (processConArgs C (rest R)))))

  (define radiogroup (lambda R
      (define bg (buttongroup))
      (for-each (lambda(b) (.add bg b)) R)
      bg))

  (define save-request (if (>= (length R) 3) (third R) (lambda R #t)))
  (define resultTA (if (null? R) (textarea 20 30 yellow) (first R)))
  (define reflect (if (>= (length R) 2) (second R) (lambda R #t)))
  (define r-apply (lambda R 
    (apply reflect R)
    (apply (eval (first R)) (cons resultTA (rest R)))))

  (define output-pane (javax.swing.JTabbedPane.))
  (define resultTA-tab
    (.add output-pane "C/C++ Output" (scrollpane resultTA)));;(textarea 10 10))))
  (define htmlpane (HP:make-HTMLPane))
  (define resultHTML-tab (scrollpane (htmlpane 'gui)))

  (define OFC (javax.swing.JFileChooser.))

  (define sourcefileTF (textfield 18))
  (define workingdirTF (textfield 18))

  ;; Java environment options
  (define javaCB (checkbox "Program (Java)"))
  (define javacCB (checkbox "Program (Javac)"))
  (define compilefilesCB (checkbox "Additional Files"))
  (define classpathCBjc (checkbox "-classpath"))
  (define sourcepathCB (checkbox "-sourcepath"))
  (define directoryCB (checkbox "-d <directory>"))
  (define deprecationCB (checkbox "-deprecation"))
  (define appletviewerCB (checkbox "appletviewer"))
  (define classpathCBj (checkbox "-cp"))
  
  (define javaTF (textfield 13))
  (define javacTF (textfield 13))
  (define compilefilesTF (textfield 13))
  (define classpathTFjc (textfield 13))
  (define sourcepathTF (textfield 13))
  (define directoryTF (textfield 13))
  (define appleturlTF (textfield 13))
  (define classpathTFj (textfield 13))
  (define mainclassTF (textfield 13))

  ;; Customized environment options
  (define customfileCB (checkbox "Source File"))
  (define customargsCB (checkbox "Arguments"))
  (define customfileCBc (checkbox "Source File"))
  (define customargsCBc (checkbox "Arguments"))
  
  (define customproTF (textfield 13))
  (define customcmdTF (textfield 13))
  (define customargsTF (textfield 13))
  (define customproTFc (textfield 13))
  (define customcmdTFc (textfield 13))
  (define customargsTFc (textfield 13))

  (define customproRB (javax.swing.JRadioButton. "Program"))
  (define customcmdRB (javax.swing.JRadioButton. "Command"))
  (define customproRBc (javax.swing.JRadioButton. "Program"))
  (define customcmdRBc (javax.swing.JRadioButton. "Command"))
  (define customRBgroup (begin (.setSelected customcmdRB #t)
			       (radiogroup customproRB customcmdRB)))
  (define customRBgroup (begin (.setSelected customcmdRBc #t)
			       (radiogroup customproRBc customcmdRBc)))

  ;; C/C++ environment options
  (define c++fileCB (checkbox "Source File"))
  (define c++argsCB (checkbox "Arguments"))
  (define c++fileCBc (checkbox "Source File"))
  (define c++argsCBc (checkbox "Arguments"))
  
  (define c++proTF (textfield 13))
  (define c++cmdTF (textfield 13))
  (define c++argsTF (textfield 13))
  (define c++proTFc (textfield 13))
  (define c++cmdTFc (textfield 13))
  (define c++argsTFc (textfield 13))

  (define c++proRB (javax.swing.JRadioButton. "Program"))
  (define c++cmdRB (javax.swing.JRadioButton. "Command"))
  (define c++proRBc (javax.swing.JRadioButton. "Program"))
  (define c++cmdRBc (javax.swing.JRadioButton. "Command"))
  (define c++RBgroup (begin (.setSelected c++cmdRB #t)
			       (radiogroup c++proRB c++cmdRB)))
  (define c++RBgroup (begin (.setSelected c++cmdRBc #t)
			       (radiogroup c++proRBc c++cmdRBc)))


  ;; Creating the Tabbed Pane
  (define (newb cb tf type option)
    (button {[type]}
        (action(lambda(e)
	    (.setSelected cb #t)
	    (writestring tf (set-option option type))))))

  (define optionsTab (javax.swing.JTabbedPane.))


  (define c++Tab
     (table 18 3
       ;; Compiling Options
       (label "") (label "Compile Options" (CourierBold 16)) (label "")
       (label "CALL" (CourierBold 14)) (label "")(label "")
       c++proRBc c++proTFc (newb c++proRBc c++proTFc 'Browse 'c++proc) 
       (label "OR" (CourierBold 14)) (label"")(label"")
       c++cmdRBc c++cmdTFc (label "")
       (label "WITH" (CourierBold 14)) (label"")(label"")
       c++argsCBc c++argsTFc (label "") 
       (label "ON" (CourierBold 14)) (label "")(label "")
       c++fileCBc (label "")(label "")
       ;; Running Options
       (label "") (label "Run/Eval Options" (CourierBold 16)) (label "")
       (label "CALL" (CourierBold 14)) (label "")(label "")
       c++proRB c++proTF (newb c++proRB c++proTF 'Browse 'c++pro)
       (label "OR" (CourierBold 14)) (label"")(label"")
       c++cmdRB c++cmdTF (label "")
       (label "WITH" (CourierBold 14)) (label"")(label"")
       c++argsCB c++argsTF (label "")       
       (label "ON" (CourierBold 14)) (label "")(label "")
       c++fileCB (label "")(label "") ))


  (define customTab
     (table 18 3
       ;; Compiling Options
       (label "") (label "Compile Options" (CourierBold 16)) (label "")
       (label "CALL" (CourierBold 14)) (label "")(label "")
       customproRBc customproTFc (newb customproRBc customproTFc
				       'Browse 'customproc) 
       (label "OR" (CourierBold 14)) (label"")(label"")
       customcmdRBc customcmdTFc (label "")
       (label "WITH" (CourierBold 14)) (label"")(label"")
       customargsCBc customargsTFc (label "") 
       (label "ON" (CourierBold 14)) (label "")(label "")
       customfileCBc (label "")(label "")
       ;; Running Options
       (label "") (label "Run/Eval Options" (CourierBold 16)) (label "")
       (label "CALL" (CourierBold 14)) (label "")(label "")
       customproRB customproTF (newb customproRB customproTF
				     'Browse 'custompro)
       (label "OR" (CourierBold 14)) (label"")(label"")
       customcmdRB customcmdTF (label "")
       (label "WITH" (CourierBold 14)) (label"")(label"")
       customargsCB customargsTF (label "")
       (label "ON" (CourierBold 14)) (label "")(label "")
       customfileCB (label "")(label "") ))


  (define javaTab
     (table 12 4       
       ;; Running Options
       (label "")(label "Runtime Options" (CourierBold 16))(label "")(label "")

       javaCB javaTF
       (newb javaCB javaTF 'Browse 'java)
       (label "")
       
       (label "Main Class") mainclassTF (label "") (label "")
       
       classpathCBj classpathTFj
       (newb classpathCBj classpathTFj 'Browse 'classpathj)
       (newb classpathCBj classpathTFj 'Add 'classpathj)
       
       appletviewerCB appleturlTF
       (newb appletviewerCB appleturlTF 'Browse 'appleturl)
       (label "")

       ;; Compiling Options
       (label "")(label "Compile Options" (CourierBold 16))(label "")(label "")

       javacCB javacTF
       (newb javacCB javacTF 'Browse 'javac)
       (label "")
       
       compilefilesCB compilefilesTF
       (newb compilefilesCB compilefilesTF 'Browse 'compilefiles)
       (newb compilefilesCB compilefilesTF 'Add 'compilefiles)
	      
       classpathCBjc classpathTFjc
       (newb classpathCBjc classpathTFjc 'Browse 'classpathjc)
       (newb classpathCBjc classpathTFjc 'Add 'classpathjc)
       
       sourcepathCB sourcepathTF
       (newb sourcepathCB sourcepathTF 'Browse 'sourcepath)
       (newb sourcepathCB sourcepathTF 'Add 'sourcepath)
       
       directoryCB directoryTF
       (newb directoryCB directoryTF 'Browse 'directory)
       (label "")
       
       deprecationCB (label "") (label "") (label "") ))


  (define options-win
    (begin
      (for-each (lambda(tf) (.setEditable tf #f))
	   (list sourcefileTF javaTF javacTF compilefilesTF classpathTFjc
		 sourcepathTF directoryTF appleturlTF classpathTFj
		 workingdirTF customproTF customproTFc c++proTF c++proTFc))
    
      (window "Coding Environment Options"
	(border (center
	  (col
	   (table 2 2
	     (label "Current Source File" (CourierItalic 15))
	     sourcefileTF
	     (button "Change Working Directory"
		     (action(lambda(e) (writestring workingdirTF
				     (set-option 'workingdir 'Browse)))))
	     workingdirTF)
       
	   (begin
	     (.add optionsTab "C/C++" c++Tab)
	     (.add optionsTab "Java" javaTab)
	     (.add optionsTab "Customized" customTab)
	     optionsTab)

	   (button "Close" (action(lambda(e) (.hide options-win))))
    )))) ))
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;END OF GUI;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


  (define (set-option option . R)
    (.setFileSelectionMode OFC 
       javax.swing.JFileChooser.FILES_AND_DIRECTORIES$)
    (.setMultiSelectionEnabled OFC #t)

    (case option
      ((sourcefile)
       (writestring workingdirTF (set! workingdir (.getParentFile (first R))))
       (cd workingdir)
       (writestring sourcefileTF (set! sourcefile (first R))))
      ((java javac custompro customproc c++pro c++proc)
       (.setMultiSelectionEnabled OFC #f)
       (.setFileSelectionMode OFC
	  javax.swing.JFileChooser.FILES_ONLY$)
       (case option
	 ((java) (set! java (filechoose OFC options-win (first R) java)))
	 ((javac) (set! javac (filechoose OFC options-win (first R) javac)))
	 ((custompro) (set! custompro
			    (filechoose OFC options-win (first R) custompro)))
	 ((customproc) (set! customproc
			     (filechoose OFC options-win (first R) customproc)))
	 ((c++pro) (set! c++pro
			 (filechoose OFC options-win (first R) c++pro)))
	 ((c++proc) (set! c++proc
			  (filechoose OFC options-win (first R) c++proc)))))
      ((compilefiles)
       (.setFileSelectionMode OFC
	  javax.swing.JFileChooser.FILES_ONLY$)
       (set! compilefiles (filechoose OFC options-win (first R) compilefiles)))
      ((classpathjc)
       (make-path (set! classpathjc 
		     (filechoose OFC options-win (first R) classpathjc))))
      ((sourcepath)
       (make-path (set! sourcepath
		     (filechoose OFC options-win (first R) sourcepath))))
      ((directory)
       (.setMultiSelectionEnabled OFC #f)
       (.setFileSelectionMode OFC
	  javax.swing.JFileChooser.DIRECTORIES_ONLY$)
       (set! directory (filechoose OFC options-win (first R) directory)))
      ((workingdir)
       (.setMultiSelectionEnabled OFC #f)
       (.setFileSelectionMode OFC
	  javax.swing.JFileChooser.DIRECTORIES_ONLY$)
       (set! workingdir (filechoose OFC options-win (first R) workingdir))
       (cd workingdir))
      ((mainclass) '...)
      ((classpathj)
       (make-path (set! classpathj
		     (filechoose OFC options-win (first R) classpathj))))
      ((appleturl)
       (.setMultiSelectionEnabled OFC #f)
       (.setFileSelectionMode OFC
	  javax.swing.JFileChooser.FILES_ONLY$)
       (set! appleturl (filechoose OFC options-win (first R) appleturl)))
    )
  )



  (define (read-n-apply buffer action)
;;    (display {\nread-n-apply [buffer] START: [(time)]\n})
    (let rloop ((mls 0L))
      ;; if ready then buffer not empty
      (if (.ready buffer)
	  ;; Iterate over the lines of a buffered reader.
	  (tryCatch
	   (let loop ((line (.readLine buffer)))
	     ;;(display {\nr-n-a [buffer]: [line]})
	     (if (and (not (eq? line #null))
		      drain-process?)
		 (begin (action line)
			(loop (.readLine buffer)))))
	   (lambda(e) (action (.toString e))))
	   ;; keep checking ready status for 5 second 
	  (if (< mls 5000L)
	      (begin (java.lang.Thread.sleep 250L) 
		     (rloop (+ mls 250L))))))
;;    (display {\nread-n-apply [buffer] END: [(time)]\n})
  )

  (define (drain r rc w wc)
    ;; Drain Reader r into Writer w.
    ;; Close the reader 
    ;; Close the writer if w
;;    (display {\ndrain [r] START: [(time)]\n})
    (read-n-apply r (lambda (L) (.println w L)))    
    (if rc (.close r))
    (if wc (.close w))
;;    (display {\ndrain [r] END: [(time)]\n})
  )


  (define (runDrainProcess realargs outputPort)
    (let* ((args (map .toString (flatten realargs)))
	   (command (.toString (which (car args)))))

      (.start (set! subprocess-thread
	  (java.lang.Thread. (lambda()
	   (tryCatch
	    (begin
	       (set! drain-process? #t) ;; enbale draining
	       (set! subprocess
		    (.exec (Runtime.getRuntime)
			   (list->array String.class (cons command (cdr args)))
			   #null ;;(array String.class)
			   (cwd)))

	       (.start (set! drain-in-thread (java.lang.Thread. (lambda()
		 (drain (inputReader subprocess) #t outputPort #f)))))

	       (.start (set! drain-err-thread (java.lang.Thread. (lambda()
		 (drain (errorReader subprocess) #t outputPort #f))))))
	    (lambda(e) (err TA {Error: [command]: command not found})))))))
;      (update-prompt 'output-writer)

      outputPort))

  (define (runDrainProcess-V realargs outputPort)
    (let* ((args (map .toString (flatten realargs)))
	   (command (.toString (which (car args)))))

      (display {\nrunDrainProcess-V START: [(time)]\n})

      (.start (set! subprocess-thread
	  (java.lang.Thread. (lambda()
	   (tryCatch
	    (begin
               (display {\n\nSubprocess Thread START: [(time)]\n\t})
	       (set! drain-process? #t) ;; enbale draining
	       (set! subprocess
		    (.exec (Runtime.getRuntime)
			   (list->array String.class (cons command (cdr args)))
			   #null ;;(array String.class)
			   (cwd)))

	       (.start (set! drain-in-thread (java.lang.Thread. (lambda()
                 (display {\ndrain-in-thread START: [(time)]\n})
		 (drain (inputReader subprocess) #t outputPort #f)
		 (display {\ndrain-in-thread END: [(time)]\n})))))

	       (.start (set! drain-err-thread (java.lang.Thread. (lambda()
                 (display {\ndrain-err-thread START: [(time)]\n})
		 (drain (errorReader subprocess) #t outputPort #f)
		 (display {\ndrain-err-thread END: [(time)]\n})))))
	     
	       (display {\nSubprocess Thread END: [(time)]\n}))
	    (lambda(e) (err TA {Error: [command]: command not found})))))))

;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;

      (display {\nrunDrainProcess-V END: [(time)]\n})

      outputPort))


  (define (is-command-alive)
    (or (.isAlive command-thread) (.isAlive subprocess-thread)
	(.isAlive drain-err-thread) (.isAlive drain-in-thread)))
  
  ;; This stops the current command call
  (define halt-command-call
    (lambda()
      (tryCatch
       (begin
	 (set! drain-process? #f)
	 (tryCatch (.destroy subprocess) (lambda(e) #t))
	 (if (.isAlive command-thread) (.stop command-thread))
	 (if (.isAlive subprocess-thread) (.stop subprocess-thread))
	 (if (.isAlive drain-err-thread) (.stop drain-err-thread))
	 (if (.isAlive drain-in-thread) (.stop drain-in-thread))
       )
       (lambda(e) (err TA {<html>ERROR in stopping command:<br><br>[
				     (.toString e)]</html>})
	 ))
;;;      (update-prompt 'shell-command)
    ))


  (define output-writer #null);;(outputWriter subprocess))

  (define prompt (textfield 12
     (action (lambda(e)
	  (r-apply '.append {[(readstring prompt)]\n})
	  (set! output-writer (outputWriter subprocess))
	  (display {[subprocess] [output-writer]})
	  (.println output-writer {[(readstring prompt)]})
	  (.flush output-writer)
	  (.setText prompt "")))))

  (define update-prompt
    (let ((shell-cmd-action (action (lambda(e)
	      (exec-cmd "Shell" (string->strlist (readstring prompt)) "")
	      (.setText prompt ""))))
	  (output-writer-action (action (lambda(e)
              (set! output-writer (outputWriter subprocess))
	      (display {[subprocess] [output-writer]})
	      (.println output-writer (readstring prompt))
	      (.flush output-writer)))))
;;      (.addActionListener prompt output-writer-action)
      (lambda (act)
	(case act
	  ((Xshell-command) (.removeActionListener prompt output-writer-action)
	   (.addActionListener prompt shell-cmd-action))
	  ((output-writer) 
	   (tryCatch
	    (begin ;;(set! output-writer (outputWriter subprocess))
		   (.removeActionListener prompt shell-cmd-action)
		   (.addActionListener prompt output-writer-action))
	    (lambda(e) (display "Err: Could not add output-writer action!")))
	   )
	  (else (display "something is wrong"))))))

  (define optionB (button "Options" (action (lambda(e)
	     (.pack options-win) (.show options-win)))))
  (define compileB (button "Compile" (action (lambda(e)
	     (exec-cmd (readstring mode-choice) 'compile sourcefile)))))
  (define runB (button "Run / Eval" (action (lambda(e)
	     (exec-cmd (readstring mode-choice) 'run sourcefile)))))
  (define stopB (button "Stop" (action (lambda(e) (halt-command-call)))))

  (define mode-choice (choice "C/C++" "Customized" "HTML" "Java" "Scheme"
     (action (lambda(e) (update-enabled-state) ))))

  (define (update-enabled-state)
    (let ((the-mode (readstring mode-choice))
	  (bstate '())
	  (change-view (lambda(title)
	    (if (not (equal? title (.getTitleAt output-pane 0)))
		(cond
		 ((equal? title "HTML Output")
		  (.remove output-pane 0)
		  ;;title icon component tip index
		  (.insertTab output-pane title #null resultHTML-tab
			      "My Output Tab" 0))
		  ;;(.addTab output-pane title resultHTML-tab))
		 ((equal? "HTML Output" (.getTitleAt output-pane 0))
		  (.remove output-pane 0)
		  (.insertTab output-pane title #null resultTA-tab
			      "My Output Tab" 0))))
		  ;;(.addTab output-pane title resultTA-tab))))
	    (.setTitleAt output-pane 0 title))) )
      (case the-mode
	(("HTML" "Scheme") (set! bstate `(,#t ,#f ,#t ,#f)))
	(("Java" "C/C++" "Customized") (set! bstate `(,#t ,#t ,#t ,#f))))
      (change-view {[the-mode] Output})
      (map .setEnabled `(,mode-choice ,compileB ,runB ,stopB) bstate)
;      (update-prompt 'shell-command)
     ))


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;;;; This is the main Scheme environment procedure
  (define (scheme-eval codestr OUTPUT outputPort)
    (define js (jscheme.JScheme.))
    (define (setPorts) 
      (let ((ev (.eval js '(Scheme.currentEvaluator))))
	(.error$# ev outputPort) ; (.setError ev outputPort)
	(.output$# ev outputPort) ; (.setOutput ev outputPort)
    ))
    (define do-setPorts (setPorts))

    (define (remote-eval js q)
      (Procedure.catching (lambda() (.eval js q)) (lambda(e) e)))

    (define (call-eval)
      (define (evalone q)
        (.eval js `(display (string-append "> " ',q "\n")))
        (let ((result (remote-eval js q)))
          (.eval js `(display (string-append ',result "\n\n")))))

      (define evalexprs
	(tryCatch 
	 (map evalone (string->exprlist codestr))
	 (lambda(e)
	   (r-apply '.setText 
	       {Evaluating Scheme...\n\nSYNTAX ERROR in \n*********\n[codestr
		     ]\n**********\n is [e]\n\n}) )))
      evalexprs
    ) ;;close call-eval

    (r-apply '.setText {[(time)]> Evaluating Scheme...\n\n})
    (call-eval)
  ) ;; close scheme-eval
  

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;;;;; This is the main Java environment procedure
  (define (java-call option file outputPort)
    (define args "no command")
    
    (cond
     ;; Compiling with Javac CONDITION
     ((equal? option 'compile)
      (save-request)
      (set! args `(,@(if (.isSelected javacCB)
			  `(,javac)
			  '(javac))
		   ,@(if (.isSelected classpathCBjc)
			 `(-classpath ,(make-path classpathjc))
			 '())
		   ,@(if (.isSelected sourcepathCB)
			 `(-sourcepath ,(make-path sourcepath))
			 '())
		   ,@(if (.isSelected directoryCB)
			 `(-d ,directory)
			     '())
		   ,@(if (.isSelected deprecationCB)
			  '(-deprecation)
			  '())
		   ,@(if (.isSelected compilefilesCB)
			 (cons sourcefile compilefiles)
			 (list sourcefile)))))
     ;; Running with appletviewer CONDITION
     ((.isSelected appletviewerCB)
        (cd (.getParent appleturl))
	(set! args `(appletviewer ,(.getName appleturl))))
     ;; Running with Java CONDITION
     ((equal? option 'run)
      (set! args `(,@(if (.isSelected javaCB)
			 `(,java)
			 '(java))
		   ,@(if (.isSelected classpathCBj)
			 `(-cp ,(make-path classpathj))
			 '())
		   ,@(string->strlist (string-append
				       (readstring mainclassTF) " "
				       (readstring prompt))) )))
    ) ;; close CONDITIONS

    (r-apply '.setText {[(time)]> [(list->spacedstr args)]\n\n})
    (runDrainProcess args outputPort)
  ) ;; close java-call


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;;;;; This is the main customized environment procedure
  (define (custom-call type file outputPort)
    (define args "no command")

    (cond
     ((equal? type 'run)
      (set! args `(,@(if (.isSelected customproRB)
			 `(,custompro)
			 (string->strlist (readstring customcmdTF)))
		   ,@(if (.isSelected customargsCB)
			 (string->strlist (readstring customargsTF)) '())
		   ,@(if (.isSelected customfileCB) `(,file) '()))))
     ((equal? type 'compile)
      (save-request)
      (set! args `(,(if (.isSelected customproRBc)
			 `(,customproc)
			 (string->strlist (readstring customcmdTFc)))
		   ,@(if (.isSelected customargsCBc)
			 (string->strlist (readstring customargsTFc)) '())
		   ,@(if (.isSelected customfileCBc) `(,file) '())))) )

    (r-apply '.setText {[(time)]> [(list->spacedstr args)]\n\n})
    (runDrainProcess args outputPort)
  ) ;; close custom-call

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;;;;; This is the main C/C++ environment procedure
  (define (c++call type file outputPort)
    (define args "no command")

    (cond
     ((equal? type 'run)
      (set! args `(,@(if (.isSelected c++proRB)
			 `(,c++pro)
			 (string->strlist (readstring c++cmdTF)))
		   ,@(if (.isSelected c++argsCB)
			 (string->strlist (readstring c++argsTF)) '())
		   ,@(if (.isSelected c++fileCB) `(,file) '()))))
     ((equal? type 'compile)
      (save-request)
      (set! args `(,(if (.isSelected c++proRBc)
			 `(,c++proc)
			 (string->strlist (readstring c++cmdTFc)))
		   ,@(if (.isSelected c++argsCBc)
			 (string->strlist (readstring c++argsTFc)) '())
		   ,@(if (.isSelected c++fileCBc) `(,file) '())))) )

    (r-apply '.setText {[(time)]> [(list->spacedstr args)]\n\n})
    (runDrainProcess args outputPort)
  ) ;; close c++call


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;;;;; This is the main environment procedure
;;   (define exec-cmd
;;      (let* 
;; 	 (
;; 	  (OUTPUT (java.io.StringWriter.))
;; 	  (outputPort (java.io.PrintWriter. OUTPUT))
;; 	  )
;;        (lambda (type option file)
;;           (.setLength (.getBuffer OUTPUT) 0)
  (define (exec-cmd type option file)
    (r-apply '.setText {[(time)]> [{[type] [option] call...}]\n\n})

     (let* 
	 (
	  (OUTPUT (java.io.StringWriter.))
	  (outputPort (java.io.PrintWriter. OUTPUT))
	  (buff (.getBuffer OUTPUT))
	  (output-thread
	   (Thread. (lambda ()

	     ;; start the command thread
	     (.start (set! command-thread
	       (Thread. (lambda ()
		 (tryCatch
		  (case type
		    (("Scheme")
		     (scheme-eval (readstring TA) OUTPUT outputPort))
		    (("Java")
		     (java-call option file outputPort))
		    (("Customized")
		     (custom-call option file outputPort))
		    (("C/C++")
		     (c++call option file outputPort))
		    (("Shell")
		     (r-apply '.setText {[(time)]> [option]\n\n})
		     (runDrainProcess option outputPort))
		    (else (throw {Bad CodingEnv Command [type]})) )
		  (lambda(e) (err TA {<html>ERROR:<br><br>[
				      (.toString e)]</html>})) )))))


	     ;; disable further command calls
	     (map .setEnabled `(,mode-choice ,compileB ,runB ,stopB) 
		  `(,#f ,#f ,#f ,#t))
;      (update-prompt 'shell-command)


	     (Thread.sleep 500L) ;; wait 1/2 sec for command-thread to execute

	     (let loop ((pos 0)
			(end (.length buff))) ;; update the output area

; 	       (.setCaretPosition resultTA
; 		     (- (.getLength (.getDocument resultTA)) 1))
	       (if (is-command-alive)
		   (begin
		     ;; WRITE OUTPUT
		     (r-apply '.append (.substring buff pos end))
		     ;; wait 1/4 sec for command-thread to execute
		     (Thread.sleep 250L)
		     (loop end (.length buff)))
		   (begin
		     ;; WRITE FINAL OUTPUT
		     (r-apply '.append (.substring buff pos end))

		     (if (.checkError outputPort)
			 (display "\n\n$#% ERROR IN THE outputPort STREAM\n\n"))
		     (.close OUTPUT)
; 		     (.setCaretPosition resultTA
; 			 (- (.getLength (.getDocument resultTA)) 1))
		     (update-enabled-state)) )))))
	 )

     (tryCatch
      (case type
	(("HTML")
	 (.start (set! command-thread (Thread. (lambda ()
	    (htmlpane 'load-string (readstring TA)))))))
	(else ;; output-thread sets and starts command-thread
	 (.start output-thread)))
      (lambda(e)
	(javax.swing.JOptionPane.showMessageDialog
	 #null
	 {<html>Cannot execute request, try saving your work.<br><br>[e]</html>}
	 "GrewpEdit"
	 javax.swing.JOptionPane.ERROR_MESSAGE$)))
     ) ;; close let*
  ) ;; close define 


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;;;;; This environment loading procedure
  (define (envload file)
    (if (not (equal? file sourcefile))
	(begin
	  (r-apply '.setEditable #f)    
	  (.setSelectedFile OFC file)
	  (set-option 'sourcefile file)

	  ;;C/C++
	  (.setSelected c++cmdRBc #t)
	  (writestring c++cmdTFc "gcc")
	  (.setSelected c++argsCBc #t)
	  (writestring c++argsTFc
		 {-o [(substring (.getName file)
				 0  (- (.length (.getName file)) 2))]})
	  (.setSelected c++fileCBc #t)
	  (.setSelected c++proRB #t)
	  (writestring c++proTF
		 {[(set! c++pro (java.io.File. (cwd)
		       (substring (.getName file)
				  0  (- (.length (.getName file)) 2))))]})

	  ;;Java
	  (writestring classpathTFj 
		(set! classpathj (make-path (list (.getParentFile file)))))
	  (writestring mainclassTF
		       (substring (.getName file)
				  0  (- (.length (.getName file)) 5))))
	(display {\n envload --> [file] == [sourcefile]\n})))


  (r-apply '.setEditable #f)
  (update-enabled-state)
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;;;;; This is the environment handler that we return
  (lambda (cmd . R)
    (case cmd
      ((load) (envload (first R)))
      ((output-pane) output-pane)
      ((control-pane)
       (col 'none 'center black
	 (row 'none 'center white
	   mode-choice compileB runB prompt stopB optionB)))))
) ;;close define




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;; This is the demo
(define (demo)
  (environment-import "groupscheme/platform/util/FileUtil.scm" "fu:")
  (letrec
     (
       (doc (textarea 10 20))
       (outTA (textarea 10 20 green))
       (outTApeer (textarea 10 20 blue))
       (update (lambda R (apply (eval (first R)) (cons outTApeer (rest R)))))
       (fc (javax.swing.JFileChooser.))
       (pep (makeCodingEnv doc outTA update))
       (readStreamIntoString
	(lambda (stream) 
	  (begin 
	    (define reader  
	      (java.io.BufferedReader. stream))
	    (define (readloop sb) 
	      (let ((x (.readLine reader))) 
		(if (equal? #null x) sb 
		    (readloop  
		     (.append  
		      (.append sb x)  
		      "\n"))))) 
	    (.toString (readloop (java.lang.StringBuffer. 8192))))))
       (win (window "Coding Environment Demo"
	      (border
	       (center (hsplit 0.25 (hsplit 0.5 (scrollpane doc)
					    (pep 'output-pane))
			       (scrollpane outTApeer)))

	       (south
		(col 
		 (pep 'control-pane)
		 (button "OPEN SOURCE FILE" (action (lambda(e)
		     (.setFileSelectionMode fc
			       javax.swing.JFileChooser.FILES_ONLY$)
		     (.setMultiSelectionEnabled fc #f)

		     (if (= (.showOpenDialog fc doc)
			    javax.swing.JFileChooser.APPROVE_OPTION$)
			 (tryCatch
			  (pep 'load
			       (let ((infile 
				      (java.io.FileReader. 
				       (java.io.File. (.getCurrentDirectory fc) 
					   (.getName (.getSelectedFile fc))))))
				 (.setText doc (readStreamIntoString infile))
				 (.validate doc)
				 (.close infile)
				 (.getSelectedFile fc))
			       'sourcefile)
			  (lambda(err) (display (list "ERROR" err))
				 (newline) #t))) ))) ))))) 
     )
  (.pack win)
  (.show win)))
