;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GrewpHelp.scm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(environment-import "groupscheme/platform/util/Util.scm" "U:")
(environment-import "jlib/Swing.scm")

(define (make-HTMLPane)
  (define htmlpane (javax.swing.JEditorPane.))
  (define history ())
  (define hist-pos -1)
  ;;returns the first n elements of the list
  (define (list-head L n)
    (reverse (list-tail (reverse L) (- (length L) n))))

  (define (history-update url)
    (set! history (append (list-head history (+ hist-pos 1)) `(,url)))
    (set! hist-pos (+ hist-pos 1)))

  (define (go direction)
    (case direction
      ((forward) (if (< hist-pos (- (length history) 1))
		     (load (list-ref history (+ hist-pos 1)) #f)))
      ((backward) (if (> hist-pos 0)
		      (load (list-ref history (- hist-pos 1)) #f)))))

  (define (load url new?)
;;    (display url)(newline)
    (U:invoke-later (lambda ()
      (tryCatch
       (let ((_url (if (string? url) (java.net.URL. url) url)))
	 (.setPage htmlpane _url)
	 (if new? (history-update _url)))
       (lambda(t)
	 (.printStackTrace t)
	 (U:error {Could not load [url]} htmlpane))))
      "Error in make-HTMLPane loading url"))

  (define (load-string html)
    (U:invoke-later (lambda ()
      (tryCatch 
       (let ((doc (.createDefaultDocument (.getEditorKit htmlpane))))
	 (.setDocument htmlpane doc)
	 (.setContentType htmlpane "text/html")
	 (.setText htmlpane html))
       (lambda(t)
	 (.printStackTrace t)
	 (U:error {Could not load [url]} htmlpane))))
      "Error in make-HTMLPane loading url"))
    

  (define link-listener
    (jsint.Listener. (lambda(e)
      (tryCatch
       (if (equal? (.getEventType e) 
		   javax.swing.event.HyperlinkEvent$EventType.ACTIVATED$)
	   (let ((pane (.getSource e)))
	     (if (.isInstance
		  javax.swing.text.html.HTMLFrameHyperlinkEvent.class
		  e)
		 (let ((doc (.getDocument pane)))
		   (.processHTMLFrameHyperlinkEvent doc e))
		 (.setPage pane (.getURL e)))
	     (history-update (.getURL e))))
       (lambda(t)
	 (.printStackTrace t)
	 (U:error {Could not load [(.getURL e)]} htmlpane))))))
  
  (.setEditable htmlpane #f)
  (.addHyperlinkListener htmlpane link-listener)

  (lambda (cmd . args)
    (case cmd
      ((load) (load (first args) #t))
      ((load-string) (load-string (first args)))
      ((forward) (go 'forward))
      ((backward) (go 'backward))
      ((gui) htmlpane))))



(define (demo)
  (define hp (make-HTMLPane))
  (define urlfield (textfield "http://groupscheme.sourceforge.net" 100))
  (define browser
    (window "Simple Browser"
      (border
        (north 
	 (row 'horizontal 'west
	  (row 'none 'west
	       (button "<-- Back" (action(lambda(e) (hp 'backward)))))
	  (row 'none 'west 
	       (button "Forward -->" (action(lambda(e) (hp 'forward)))))
	  urlfield
	  (row 'none 'west
	       (button "GO" (action(lambda(e) 
				     (hp 'load (readstring urlfield))))))))
	(center (scrollpane (hp 'gui))))))
  (define link-listener
    (jsint.Listener. (lambda(e)
      (tryCatch
       (if (equal? (.getEventType e) 
		   javax.swing.event.HyperlinkEvent$EventType.ACTIVATED$)
	   (writestring urlfield (.toString (.getURL e))))
       (lambda(t)
	 (writestring urlfield {Could not load [(.getURL e)]}))))))

  (.addHyperlinkListener (hp 'gui) link-listener)
  (hp 'load (readstring urlfield))
  (.resize browser 800 600)
  (.show browser))