;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Currently each binding must have a unique name even if it
;; corresponds to an existing action. Also, these KeyStrokes
;; are loosely based on an english keyborad (hint PowerBook G4)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(import "javax.swing.text.DefaultEditorKit")
(environment-import "groupscheme/platform/util/bindings/simple.scm" "s:")
(environment-import "groupscheme/platform/util/bindings/kill-ring.scm" "kr:")
(environment-import "groupscheme/platform/util/bindings/search-replace.scm"
		    "sr:")



(define defaults
  (list
   ;; Name                Procedure
   ;; ---                 -------
   `(goto-line            ,s:goto-line)
   `(emacs-undo           ,s:emacs-undo)
   `(show-help            ,s:show-help)
   `(kill-line            ,kr:kill-line)
   `(kill-region          ,kr:kill-region)
   `(yank                 ,kr:yank)
   `(query-replace        ,sr:query-replace)
   `(isearch              ,sr:isearch)
   `(isearch-backward     ,sr:isearch-backward)
  ) ;;close list
) ;;close define





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A list of Names that correspond to Actions that are ;;
;; already available in Java.  Thus, simply adding a   ;;
;; Name from this list to a TextComponent's InputMap   ;;
;; with a binding KeyStroke will execute the action.   ;;
;; i.e. (.put (.getInputMap TC) KeyStroke Name)        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define java-defaults
  (list
   DefaultEditorKit.backwardAction$
   DefaultEditorKit.beepAction$
   DefaultEditorKit.beginAction$
   DefaultEditorKit.beginLineAction$
   DefaultEditorKit.beginParagraphAction$
   DefaultEditorKit.beginWordAction$
   DefaultEditorKit.copyAction$
   DefaultEditorKit.cutAction$
   DefaultEditorKit.defaultKeyTypedAction$
   DefaultEditorKit.deleteNextCharAction$
   DefaultEditorKit.deletePrevCharAction$
   DefaultEditorKit.downAction$
   DefaultEditorKit.endAction$
   DefaultEditorKit.endLineAction$
   DefaultEditorKit.EndOfLineStringProperty$
   DefaultEditorKit.endParagraphAction$
   DefaultEditorKit.endWordAction$
   DefaultEditorKit.forwardAction$
   DefaultEditorKit.insertBreakAction$
   DefaultEditorKit.insertContentAction$
   DefaultEditorKit.insertTabAction$
   DefaultEditorKit.nextWordAction$
   DefaultEditorKit.pageDownAction$
   DefaultEditorKit.pageUpAction$
   DefaultEditorKit.pasteAction$
   DefaultEditorKit.previousWordAction$
   DefaultEditorKit.readOnlyAction$
   DefaultEditorKit.selectAllAction$
   DefaultEditorKit.selectionBackwardAction$
   DefaultEditorKit.selectionBeginAction$
   DefaultEditorKit.selectionBeginLineAction$
   DefaultEditorKit.selectionBeginParagraphAction$
   DefaultEditorKit.selectionBeginWordAction$
   DefaultEditorKit.selectionDownAction$
   DefaultEditorKit.selectionEndAction$
   DefaultEditorKit.selectionEndLineAction$
   DefaultEditorKit.selectionEndParagraphAction$
   DefaultEditorKit.selectionEndWordAction$
   DefaultEditorKit.selectionForwardAction$
   DefaultEditorKit.selectionNextWordAction$
   DefaultEditorKit.selectionPreviousWordAction$
   DefaultEditorKit.selectionUpAction$
   DefaultEditorKit.selectLineAction$
   DefaultEditorKit.selectParagraphAction$
   DefaultEditorKit.selectWordAction$
   DefaultEditorKit.upAction$
   DefaultEditorKit.writableAction$
  )
)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Action Names With Description                         ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define java-description-defaults
  `(
    (,DefaultEditorKit.backwardAction$ "Name of the Action for moving the caret logically backward one position.")
    (,DefaultEditorKit.beepAction$ "Name of the action to create a beep.")
    (,DefaultEditorKit.beginAction$ "Name of the Action for moving the caret to the beginning of the document.")
    (,DefaultEditorKit.beginLineAction$ "Name of the Action for moving the caret to the beginning of a line.")
    (,DefaultEditorKit.beginParagraphAction$ "Name of the Action for moving the caret to the beginning of a paragraph.")
    (,DefaultEditorKit.beginWordAction$ "Name of the Action for moving the caret to the beginning of a word.")
    (,DefaultEditorKit.copyAction$ "Name of the action to copy the selected region and place the contents into the system clipboard.")
    (,DefaultEditorKit.cutAction$ "Name of the action to cut the selected region and place the contents into the system clipboard.")
    (,DefaultEditorKit.defaultKeyTypedAction$ 
      "Name of the action that is executed by default if a key typed "
      "event is received and there is no keymap entry.")
    (,DefaultEditorKit.deleteNextCharAction$ "Name of the action to delete the character of content that follows the current caret position.")
    (,DefaultEditorKit.deletePrevCharAction$ "Name of the action to delete the character of content that precedes the current caret position.")
    (,DefaultEditorKit.downAction$ "Name of the Action for moving the caret logically downward one position.")
    (,DefaultEditorKit.endAction$ "Name of the Action for moving the caret to the end of the document.")
    (,DefaultEditorKit.endLineAction$ "Name of the Action for moving the caret to the end of a line.")
    (,DefaultEditorKit.EndOfLineStringProperty$ {When reading a document if a CRLF is encountered a property with this name is added and the value will be "\r\n".})
    (,DefaultEditorKit.endParagraphAction$ "Name of the Action for moving the caret to the end of a paragraph.")
    (,DefaultEditorKit.endWordAction$ "Name of the Action for moving the caret to the end of a word.")
    (,DefaultEditorKit.forwardAction$ "Name of the Action for moving the caret logically forward one position.")
    (,DefaultEditorKit.insertBreakAction$ "Name of the action to place a line/paragraph break into the document.")
    (,DefaultEditorKit.insertContentAction$ "Name of the action to place content into the associated document.")
    (,DefaultEditorKit.insertTabAction$ "Name of the action to place a tab character into the document.")
    (,DefaultEditorKit.nextWordAction$ "Name of the Action for moving the caret to the beginning of the next word.")
    (,DefaultEditorKit.pageDownAction$ "Name of the action to page down vertically.")
    (,DefaultEditorKit.pageUpAction$ "Name of the action to page up vertically.")
    (,DefaultEditorKit.pasteAction$ "Name of the action to paste the contents of the system clipboard into the selected region, or before the caret if nothing is selected.")
    (,DefaultEditorKit.previousWordAction$ "Name of the Action for moving the caret to the beginning of the previous word.")
    (,DefaultEditorKit.readOnlyAction$ "Name of the action to set the editor into read-only mode.")
    (,DefaultEditorKit.selectAllAction$ "Name of the Action for selecting the entire document")
    (,DefaultEditorKit.selectionBackwardAction$ "Name of the Action for extending the selection by moving the caret logically backward one position.")
    (,DefaultEditorKit.selectionBeginAction$ "Name of the Action for moving the caret to the beginning of the document.")
    (,DefaultEditorKit.selectionBeginLineAction$ "Name of the Action for moving the caret to the beginning of a line, extending the selection.")
    (,DefaultEditorKit.selectionBeginParagraphAction$ "Name of the Action for moving the caret to the beginning of a paragraph, extending the selection.")
    (,DefaultEditorKit.selectionBeginWordAction$ "Name of the Action for moving the caret to the beginning of a word, extending the selection.")
    (,DefaultEditorKit.selectionDownAction$ "Name of the Action for moving the caret logically downward one position, extending the selection.")
    (,DefaultEditorKit.selectionEndAction$ "Name of the Action for moving the caret to the end of the document.")
    (,DefaultEditorKit.selectionEndLineAction$ "Name of the Action for moving the caret to the end of a line, extending the selection.")
    (,DefaultEditorKit.selectionEndParagraphAction$ "Name of the Action for moving the caret to the end of a paragraph, extending the selection.")
    (,DefaultEditorKit.selectionEndWordAction$ "Name of the Action for moving the caret to the end of a word, extending the selection.")
    (,DefaultEditorKit.selectionForwardAction$ "Name of the Action for extending the selection by moving the caret logically forward one position.")
    (,DefaultEditorKit.selectionNextWordAction$ "Name of the Action for moving the selection to the beginning of the next word, extending the selection.")
    (,DefaultEditorKit.selectionPreviousWordAction$ "Name of the Action for moving the selection to the beginning of the previous word, extending the selection.")
    (,DefaultEditorKit.selectionUpAction$ "Name of the Action for moving the caret logically upward one position, extending the selection.")
    (,DefaultEditorKit.selectLineAction$ "Name of the Action for selecting a line around the caret.")
    (,DefaultEditorKit.selectParagraphAction$ "Name of the Action for selecting a paragraph around the caret.")
    (,DefaultEditorKit.selectWordAction$ "Name of the Action for selecting a word around the caret.")
    (,DefaultEditorKit.upAction$ "Name of the Action for moving the caret logically upward one position.")
    (,DefaultEditorKit.writableAction$ "Name of the action to set the editor into writeable mode.")
  )
)
