(environment-import "jlib/Swing.scm")

(define (isearch bindings TC)
  (isearching 'forward bindings TC))

(define (isearch-backward bindings TC)
  (isearching 'backward bindings TC))

(define (isearching direction bindings TC)
  (letrec
    ((current-pos ())
     (search
      (lambda (pos) (invoke (lambda()
	(if (and (.isInstance javax.swing.text.Position.class pos)
		 (not (equal? "" (readstring (bindings 'tag 'buffer)))))
	    (let* ((query (readstring (bindings 'tag 'buffer)))
		   (newpos (find TC direction query pos)))
	      (if newpos
		  (let ((start (.getOffset newpos))
			(end (+ (.getOffset newpos) (.length query))))
		    (highlighter bindings TC 'clear null null)
		    (.setCaretPosition TC (if (eq? direction 'forward)
					      end start))
		    (writestring (bindings 'tag 'status) 
			 {I-Search[(if (eq? direction 'backward)
				       " backward:" ":")]})
		    (highlighter bindings TC 'add 
				 `((,start ,end)) (make-painter 200 0 0))
		    (highlighter bindings TC 'add (find TC 'all query pos) 
				 (make-painter 0 0 150)))
		  (writestring (bindings 'tag 'status) 
		       {Failing I-Search[(if (eq? direction 'backward)
					     " backward:" ":")]})))
	    (highlighter bindings TC 'clear null null))))))

     (focus-listener
      (jsint.Listener. (lambda(e)
        (if (= (.getID e) java.awt.event.FocusEvent.FOCUS_GAINED$)
	    (exit)))))

     (incremental-listener
      (jsint.Listener. (lambda(e)
	(search current-pos))))

     (quit
      (lambda R (invoke (lambda()
	 (.setCaretPosition TC
	     (.getOffset (bindings 'tag 'search-start-position)))
	 (exit)))))

     (exit
      (lambda R (invoke (lambda()
	   (bindings 'tag 'search-start-position ())
	   (bindings 'tag 'last-search-query 
		     (readstring (bindings 'tag 'buffer)))
	   (.removeFocusListener TC focus-listener)
	   (.removeDocumentListener (.getDocument (bindings 'tag 'buffer)) 
				    incremental-listener)
	   (highlighter bindings TC 'clear null null)
	   (IS 'remove (bindings 'tag 'buffer) "")
	   (.setEnabled (bindings 'tag 'buffer) #f)
	   (.requestFocusInWindow TC)))))

     (IS (lambda (act buffer status)
	   (bindings act buffer 'eis-exit '(ESCAPE) exit)
	   (bindings act buffer 'gis-exit '(CTRL G) quit)
	   (bindings act buffer 'is-enter '(ENTER) isearch-procedure)
	   (bindings act buffer 'ris-enter '(CTRL ENTER) inverse-isearch)
	   (writestring (bindings 'tag 'status) status)
	   (writestring buffer "")))

     (inverse-isearch
      (lambda R
	(set! direction (if (eq? direction 'forward) 'backward 'forward))
	(isearch-procedure)))

     (isearch-procedure
      (lambda R (invoke (lambda()
	;; if null then new search
	(if (null? (bindings 'tag 'search-start-position))
	    (begin
	      (.setEnabled (bindings 'tag 'buffer) #t)
	      (.requestFocusInWindow (bindings 'tag 'buffer))
	      (IS 'add (bindings 'tag 'buffer) 
		  {I-Search[(if (eq? direction 'backward)
				" backward:" ":")]})
	      (.addFocusListener TC focus-listener)
	      (.addDocumentListener (.getDocument (bindings 'tag 'buffer))
				    incremental-listener)
	      (search (set! current-pos
		(bindings 'tag 'search-start-position
		  (.createPosition (.getDocument TC) 
				   (.getCaretPosition TC))))))
	    (begin ;; if searching for nothing?
	      (if (equal? "" (readstring (bindings 'tag 'buffer)))
		  (writestring (bindings 'tag 'buffer)
			       (bindings 'tag 'last-search-query)))
	      (if (.startsWith (readstring (bindings 'tag 'status)) 
			       "Failing")
		  (if (eq? direction 'forward)
		      (search (.getStartPosition (.getDocument TC)))
		      (search (.getEndPosition (.getDocument TC))))
		   (search (set! current-pos
		     (.createPosition (.getDocument TC) 
				      (.getCaretPosition TC)))))))))))
      );; close letrec defines

    (bindings 'tag 'last-search-query "")
    (bindings 'tag 'search-start-position ())
    isearch-procedure
 );; close letrec
);; close define



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (query-replace bindings TC)
  (letrec
      ((respond (lambda (response)
          (invoke (lambda ()
	     (cond 
	      ((.equalsIgnoreCase "y" response)
	       (.replaceSelection TC 
		   (bindings 'tag 'query-replace-replacement))
	       (if (not (replace bindings TC 'next))
		   (exit)))
	      ((.equalsIgnoreCase "n" response)
	       (if (not (replace bindings TC 'next))
		   (exit)))
	      ((.equalsIgnoreCase "r" response)
	       (replace bindings TC 'remaining)
	       (exit))
	      ((.equalsIgnoreCase "a" response)
	       (replace bindings TC 'all)
	       (exit))
	      ((.equalsIgnoreCase "q" response)
	       (exit))
	      ((equal? "?" response)
	       (writestring (bindings 'tag 'buffer) "")
	       (javax.swing.JOptionPane.showMessageDialog TC
		{[{<html>When replacing the query string:<br><br>}
		  {(Y)es: replaces the highlighted query<br>}
		  {(N)o: skips to the next query match<br>}
		  {(R)emaining: replaces all subsequent query matches<br>}
		  {(A)ll: replaces all the existing query matches<br>}
		  {(Q)uit: exits query-replace mode</html}]}
		"Bindings"
		javax.swing.JOptionPane.INFORMATION_MESSAGE$))
	      (else
	       (display "not a valid response\n")))))))

       (focus-listener
	(jsint.Listener. (lambda(e)
           (if (= (.getID e) java.awt.event.FocusEvent.FOCUS_GAINED$)
	       (exit)))))

       (response-listener
	(jsint.Listener. (lambda(e)
           (respond (readstring (bindings 'tag 'buffer))))))

       (enter
	(lambda R (invoke (lambda()
	  (if (null? (bindings 'tag 'query-replace-query))
	      (begin 
		(bindings 'tag 'query-replace-query
			  (readstring (bindings 'tag 'buffer)))
		(writestring (bindings 'tag 'status)
		  {Query replace [(bindings 'tag 'query-replace-query)] with:})
		(writestring (bindings 'tag 'buffer) ""))
	      (if (null? (bindings 'tag 'query-replace-replacement))
		  (begin
		    (bindings 'tag 'query-replace-replacement 
			      (readstring (bindings 'tag 'buffer)))
		    (.addFocusListener TC focus-listener)
		    (.addDocumentListener 
		       (.getDocument (bindings 'tag 'buffer))
		       response-listener)
		    (if (not (replace bindings TC 'next))
			(exit)))
		  (exit)))))))

       (exit
	(lambda R (invoke (lambda ()
	   (highlighter bindings TC 'clear null null)
	   (.removeFocusListener TC focus-listener)
	   (.removeDocumentListener
	    (.getDocument (bindings 'tag 'buffer))
	    response-listener)
	   (QR 'remove (bindings 'tag 'buffer) "")
	   (.setEnabled (bindings 'tag 'buffer) #f)
	   (.requestFocusInWindow TC)))))

       (QR (lambda (act buffer status)
	     (bindings 'tag 'query-replace-query ())
	     (bindings 'tag 'query-replace-replacement ())
	     (bindings act buffer 'eqr-exit '(ESCAPE) exit)
	     (bindings act buffer 'gqr-exit '(CTRL G) exit)
	     (bindings act buffer 'qr-enter '(ENTER) enter)
	     (writestring (bindings 'tag 'status) status)
	     (writestring buffer "")))
      );; close letrec defines

    (lambda R (invoke (lambda ()
       (.setEnabled (bindings 'tag 'buffer) #t)
       (.requestFocusInWindow (bindings 'tag 'buffer))
       (QR 'add (bindings 'tag 'buffer) "Query Replace:"))))

 );; close letrec
);; close define


;; Might want to switch .replaceSelection with .replaceRegion because of
;; possible selection loss when other users type in shared TC
(define (replace bindings TC type)
  (let ((pos (lambda() (.createPosition
			(.getDocument TC) (.getCaretPosition TC))))
	(_query (bindings 'tag 'query-replace-query))
	(_replacement (bindings 'tag 'query-replace-replacement)))
    (case type
      ((next)
       (let ((rpos (find TC 'forward _query (pos))))
	 (if rpos
	     (let ((start (.getOffset rpos))
		   (end (+ (.getOffset rpos) (.length _query))))
	       (.select TC start end)
	       (highlighter bindings TC 'clear null null)
	       (highlighter bindings TC 'add `((,start ,end)) 
			    (make-painter 255 0 0))
	       (writestring (bindings 'tag 'status)
		  {Query replace [_query] with [_replacement]: (? for help)})
	       (writestring (bindings 'tag 'buffer) "")))))
      ((remaining)
	  (.replaceSelection TC _replacement)
	  (let loop ((rpos (find TC 'forward _query (pos))))
	    (if rpos
		(begin (replace bindings TC 'next)
		       (.replaceSelection TC _replacement)
		       (loop (find TC 'forward _query (pos)))))))
      ((all)
       (let ((occurrences (find TC 'all _query null)))
	 (if (not (null? occurrences))
	       (map (lambda(p) (.replaceRange TC _replacement 
					      (first p) (second p)))
		    occurrences)))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (invoke lambda-expr)
  (java.awt.EventQueue.invokeLater
   (tryCatch 
    lambda-expr
    (lambda(e) 
      (javax.swing.JOptionPane.showMessageDialog
       #null {Search-Replace Invoke Error: [e]} "Bindings"
       javax.swing.JOptionPane.ERROR_MESSAGE$)))))

(define (find TC type str pos)
  (let ((text (if (.equals str (.toLowerCase str))
		  (.toLowerCase (readstring TC))
		  (readstring TC)))
	(index -1))
    (case type
      ((forward iforward)
       (set! index (.indexOf text str (.getOffset pos)))
       (if (!= index -1) (.createPosition (.getDocument TC) index)))
      ((backward ibackward)
       (set! index (.lastIndexOf text str (- (.getOffset pos) 1)))
       (if (!= index -1) (.createPosition (.getDocument TC) index)))
      ((all)
       (let* ((pattern (java.util.regex.Pattern.compile {\\Q[str]\\E}))
	      (matcher (.matcher pattern text)))
	 (let loop ((match? (.find matcher))
		    (occurrences '()))
	   (if match? ;;must get the match before calling .find
	       (let ((match `(,(.start matcher) ,(.end matcher))))
		 (loop (.find matcher) (cons match occurrences)))
	       occurrences)))))))

(define (make-painter r g b)
  (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter.
   (color r g b 128)))
(define (highlighter bindings TC cmd data painter)
  (let ((HL (.getHighlighter TC)))
    (bindings 'tag 'highlights
       (case cmd
	 ((add)
	  (append (bindings 'tag 'highlights)
		  (map (lambda (os)
			 (.addHighlight HL (first os) (second os) painter))
		       data)))
	 ((clear)
	  (for-each (lambda (key) (.removeHighlight HL key))
		    (bindings 'tag 'highlights)))))))