package groupscheme.platform.wrapper;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import javax.swing.text.Segment;

import jsint.Procedure;
import jscheme.JS;

public class PlainDocumentWrapper extends PlainDocument {

    Procedure insertProc, deleteProc;
    JS js;

    public PlainDocumentWrapper(Procedure insertProc, Procedure deleteProc){
	super();
	this.insertProc = insertProc;
	this.deleteProc = deleteProc;
        this.js = new JS();
    }

    /**
     * This is called when someone attempts to insert a string in the textarea based on this document.
     * It does not read or modify the document at all, rather it calls the insertProc on the arguments
     **/
    synchronized public void insertString(int offset, String str, AttributeSet a) throws BadLocationException {
	//	System.out.println("enter insertString:"+offset+":"+str+":"+this);
	if (insertProc != null)
	    js.call(insertProc, new Integer(offset), str, a);
	//	System.out.println(" exit insertString:"+offset+":"+str+":"+this);
    }


    /**
     * This is called when someone attempts to delete a string from the textarea based on this document.
     * It does not read or modify the document at all, rather it calls the deleteProc on the arguments
     **/
    synchronized public void remove(int offset, int len) throws BadLocationException {
	//	System.out.println("enter deleteString:"+offset+":"+len+":"+this);
	Segment s = new Segment();
        this.getText(offset,len,s);
	if (deleteProc != null)
	    js.call(deleteProc, new Integer(offset),new Integer(len),s);
	//	System.out.println(" exit deleteString:"+offset+":"+len+":"+this);
    }


    /**
     * this is an alias for the insertString of the superclass. It is called when we really want to
     * insert something into the textarea. The 'Procedure' argument is deprecated and is no longer used.
     * This grabs a write lock before inserting. According to the Java docs, t is thread-safe.
     **/
    synchronized public void insertStringNoSend(int offset, String str, AttributeSet a, Procedure p) throws BadLocationException {
	//	System.out.println("enter NOSEND insertString:"+offset+":"+str+":"+this);
        super.insertString(offset,str,a);
	if (p != null) 
	    js.call(p, new Integer(offset),str,a);
	//	System.out.println(" exit NOSEND insertString:"+offset+":"+str+":"+this);
    }


    /**
     * this is an alias for the insertString of the superclass. It is called when we really want to
     * insert something into the textarea. The 'Procedure' argument is deprecated and is no longer used
     * This grabs a write lock before deleting. According to the Java docs, t is thread-safe.
     **/
    synchronized public void removeNoSend(int offset, int len,Procedure p) throws BadLocationException {
	//	System.out.println("enter NOSEND deleteString:"+offset+":"+len+":"+this);
        super.remove(offset,len);
	if (p != null) 
	    js.call(p, new Integer(offset),new Integer(len));
	//	System.out.println(" exit NOSEND deleteString:"+offset+":"+len+":"+this);
    }
}
